//this file is part of eMule
//Copyright (C)2002-2005 Merkur ( devs@emule-project.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#include "stdafx.h"
#include "emule.h"
#include "AntiLeechListCtrl.h"
#include "otherfunctions.h"
#include "MenuCmds.h"
#include "MemDC.h"
#include "emuleDlg.h"
#include "Log.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

CString GetYesNo(const bool& b)	
{
	if(b)
		return GetResString(IDS_YES);
	return GetResString(IDS_NO);
}

CLeecherData::CLeecherData(const CString& leecher, const uint8& type, const CString& reason, const bool& ban, const bool& bcase)
{
	strLeecher = leecher;
	uiType = type;
	strReason = reason;
	bBanned = ban;
	bCase = bcase;
}

// CAntiLeechListCtrl
IMPLEMENT_DYNAMIC(CAntiLeechListCtrl, CMuleListCtrl)
BEGIN_MESSAGE_MAP(CAntiLeechListCtrl, CMuleListCtrl)
	ON_WM_CONTEXTMENU()
	ON_WM_SYSCOLORCHANGE()
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnNMDblclk)
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetDispInfo)
	ON_NOTIFY_REFLECT(LVN_GETINFOTIP, OnLvnGetInfoTip)
END_MESSAGE_MAP()

CAntiLeechListCtrl::CAntiLeechListCtrl()
{
	SetGeneralPurposeFind(true); //>>> WiZaRd - allow search
}

void CAntiLeechListCtrl::Init()
{
	SetName(_T("AntiLeechListCtrl"));
	CImageList ilDummyImageList; //dummy list for getting the proper height of listview entries
	ilDummyImageList.Create(1, theApp.GetSmallSytemIconSize().cy,theApp.m_iDfltImageListColorFlags|ILC_MASK, 1, 1); 
	SetImageList(&ilDummyImageList, LVSIL_SMALL);
	ASSERT( (GetStyle() & LVS_SHAREIMAGELISTS) == 0 );
	ilDummyImageList.Detach();

	ModifyStyle(LVS_SINGLESEL, 0);
	SetExtendedStyle(LVS_EX_FULLROWSELECT /*| LVS_EX_INFOTIP*/ | LVS_EX_CHECKBOXES | LVS_EX_GRIDLINES);

	InsertColumn(0,_T("ToBan"), LVCFMT_LEFT, 150);
	InsertColumn(1,_T("Kind"), LVCFMT_LEFT, 100);
	InsertColumn(2,_T("Reason"), LVCFMT_LEFT, 200);
	InsertColumn(3,_T("Banned"), LVCFMT_LEFT, 50);
	InsertColumn(4,_T("Case"), LVCFMT_LEFT, 50);

	SetAllIcons();
	Localize();
	LoadSettings();

	// Barry - Use preferred sort order from preferences
	SetSortArrow();
	SortItems(SortProc, GetSortItem() + (GetSortAscending() ? 0:100));
}

CAntiLeechListCtrl::~CAntiLeechListCtrl()
{
}

void CAntiLeechListCtrl::OnSysColorChange()
{
	CMuleListCtrl::OnSysColorChange();
	SetAllIcons();
}

void CAntiLeechListCtrl::SetAllIcons()
{
	imagelist.DeleteImageList();
	imagelist.Create(16,16,theApp.m_iDfltImageListColorFlags|ILC_MASK,0,1);
	imagelist.SetBkColor(CLR_NONE);
	imagelist.Add(CTempIconLoader(_T("ENEMY")));
	imagelist.Add(CTempIconLoader(_T("SRCDOWNLOADING")));
}

void CAntiLeechListCtrl::Localize()
{
	CHeaderCtrl* pHeaderCtrl = GetHeaderCtrl();
	HDITEM hdi;
	hdi.mask = HDI_TEXT;
	CString strRes;

	for (int icol=0;icol< pHeaderCtrl->GetItemCount() ;icol++) 
	{
		switch (icol) 
		{
			case 0: strRes = _T("ToBan"); break;
			case 1: strRes = _T("Kind"); break;
			case 2: strRes = _T("Reason"); break;
			case 3: strRes = _T("Banned"); break;
			case 4: strRes = _T("Case"); break;	
		}
		strRes.Remove(':');
		hdi.pszText = strRes.GetBuffer();
		pHeaderCtrl->SetItem(icol, &hdi);
		strRes.ReleaseBuffer();
	}
}

void CAntiLeechListCtrl::AddLeecher(const CLeecherData* leecher)
{
	if (!theApp.emuledlg->IsRunning())
		return;

	int iItemCount = GetItemCount();
	int iItem = InsertItem(LVIF_TEXT|LVIF_PARAM,iItemCount,LPSTR_TEXTCALLBACK,0,0,0,(LPARAM)leecher);
	Update(iItem);
	SetCheck(iItem, leecher->bBanned);
}

void CAntiLeechListCtrl::RemoveLeecher(const CLeecherData* leecher)
{
	if (!theApp.emuledlg->IsRunning())
		return;

	LVFINDINFO find;
	find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)leecher;
	sint32 result = FindItem(&find);
	if (result != -1){
		DeleteItem(result);
	}
	delete leecher;
}

void CAntiLeechListCtrl::RefreshLeecher(const CLeecherData* leecher)
{
	// There is some type of timing issue here.. If you click on item in the queue or upload and leave
	// the focus on it when you exit the cient, it breaks on line 854 of emuleDlg.cpp
	// I added this IsRunning() check to this function and the DrawItem method and
	// this seems to keep it from crashing. This is not the fix but a patch until
	// someone points out what is going wrong.. Also, it will still assert in debug mode..
	if (!theApp.emuledlg->IsRunning())
		return;

	LVFINDINFO find;
	find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)leecher;
	sint32 result = FindItem(&find); //FiX? sint16
	if (result != -1)
	{
		Update(result);
		SetCheck(result, leecher->bBanned);
	}
}

#define DLC_DT_TEXT (DT_LEFT|DT_SINGLELINE|DT_VCENTER|DT_NOPREFIX|DT_END_ELLIPSIS)

void CAntiLeechListCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	if( !theApp.emuledlg->IsRunning() )
		return;
	if (!lpDrawItemStruct->itemData)
		return;

	CDC* odc = CDC::FromHandle(lpDrawItemStruct->hDC);
	BOOL bCtrlFocused = ((GetFocus() == this ) || (GetStyle() & LVS_SHOWSELALWAYS));
	const CLeecherData* leecher = (CLeecherData*)lpDrawItemStruct->itemData;
	if (lpDrawItemStruct->itemState & ODS_SELECTED)
	{
		if(bCtrlFocused)
			odc->SetBkColor(m_crHighlight);
		else
			odc->SetBkColor(m_crNoHighlight);
	}
	else
		odc->SetBkColor(GetBkColor());

	CMemDC dc(odc, &lpDrawItemStruct->rcItem);
	CFont* pOldFont = dc.SelectObject(GetFont());
	CRect cur_rec(lpDrawItemStruct->rcItem);
	//COLORREF crOldTextColor = dc.SetTextColor((lpDrawItemStruct->itemState & ODS_SELECTED) ? m_crHighlightText : m_crWindowText);
	COLORREF crOldTextColor;
	if(!leecher->bBanned) // Aireoreion: other way
		crOldTextColor = dc->SetTextColor(RGB(0, 128, 0));
	else
		crOldTextColor = dc->SetTextColor(RGB(255, 0, 0));
		
	COLORREF crOldBackColor = odc->GetBkColor(); 

	int iOldBkMode;
	if (m_crWindowTextBk == CLR_NONE){
		DefWindowProc(WM_ERASEBKGND, (WPARAM)(HDC)dc, 0);
		iOldBkMode = dc.SetBkMode(TRANSPARENT);
	}
	else
		iOldBkMode = OPAQUE;

	CHeaderCtrl *pHeaderCtrl = GetHeaderCtrl();
	int iCount = pHeaderCtrl->GetItemCount();
	cur_rec.right = cur_rec.left - 8;
	cur_rec.left += 4;

	CString Sbuffer = _T("");	
	for (int iCurrent = 0; iCurrent < iCount; iCurrent++)
	{
		int iColumn = pHeaderCtrl->OrderToIndex(iCurrent);
		if (!IsColumnHidden(iColumn))
		{
			cur_rec.right += GetColumnWidth(iColumn);
			switch (iColumn)
			{
				case 0:
				{						
					POINT point = {cur_rec.left,cur_rec.top+1};
					if(leecher->bBanned)
						imagelist.Draw(dc, 0, point, ILD_NORMAL);						
					else 
						imagelist.Draw(dc, 1, point, ILD_NORMAL);
					cur_rec.left += 16+2;
					
					Sbuffer = leecher->strLeecher;
					dc.DrawText(Sbuffer,Sbuffer.GetLength(),&cur_rec,DLC_DT_TEXT);
					cur_rec.left -= 16+2;
					break;					
				}
				case 1:
					switch(leecher->uiType)
					{
						case 0:
							Sbuffer = _T("Name");
							break;
						case 1:
							Sbuffer = _T("Mod");
							break;
						case 2:
							Sbuffer = _T("Ver");
							break;
						default:
							Sbuffer.Empty();
							break;
					}
					break;
				case 2:
					Sbuffer = leecher->strReason;
					break;
				case 3:
					Sbuffer.Format(_T("%s"), GetYesNo(leecher->bBanned));
					break;
				case 4:
					Sbuffer.Format(_T("%s"), GetYesNo(leecher->bCase));
					break;
			}

			if( iColumn != 0 )
				dc.DrawText(Sbuffer,Sbuffer.GetLength(),&cur_rec,DLC_DT_TEXT);
			cur_rec.left += GetColumnWidth(iColumn);
			dc.SetBkColor(crOldBackColor);
		}
	}

	//draw rectangle around selected item(s)
	if (lpDrawItemStruct->itemState & ODS_SELECTED)
	{
		RECT outline_rec = lpDrawItemStruct->rcItem;

		outline_rec.top--;
		outline_rec.bottom++;
		dc.FrameRect(&outline_rec, &CBrush(GetBkColor()));
		outline_rec.top++;
		outline_rec.bottom--;
		outline_rec.left++;
		outline_rec.right--;

		if(bCtrlFocused)
			dc.FrameRect(&outline_rec, &CBrush(m_crFocusLine));
		else
			dc.FrameRect(&outline_rec, &CBrush(m_crNoFocusLine));
	}

	if (m_crWindowTextBk == CLR_NONE)
		dc.SetBkMode(iOldBkMode);
	dc.SelectObject(pOldFont);
	dc.SetTextColor(crOldTextColor);
}

// CAntiLeechListCtrl message handlers
void CAntiLeechListCtrl::OnContextMenu(CWnd* pWnd, CPoint point)
{
}

BOOL CAntiLeechListCtrl::OnCommand(WPARAM wParam,LPARAM lParam )
{
	return true;
}


void CAntiLeechListCtrl::OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult)
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	// if it's a second click on the same column then reverse the sort order,
	// otherwise sort the new column in ascending order.

	// Barry - Store sort order in preferences
	// Determine ascending based on whether already sorted on this column
	bool sortAscending = (GetSortItem() != pNMListView->iSubItem) ? true : !GetSortAscending();

	// Sort table
	UpdateSortHistory(pNMListView->iSubItem + (sortAscending ? 0:100));
	SetSortArrow(pNMListView->iSubItem, sortAscending);
	SortItems(SortProc, pNMListView->iSubItem + (sortAscending ? 0:100));

	*pResult = 0;
}

int CAntiLeechListCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	const CLeecherData* item1 = (CLeecherData*)lParam1;
	const CLeecherData* item2 = (CLeecherData*)lParam2;

	if(lParamSort > 99)
	{
		const CLeecherData* temp = item1;
		item1 = item2;
		item2 = temp;
		lParamSort -= 100;
	}

	int iResult = 0;
	switch(lParamSort)
	{
		case 0: 
			iResult = CompareLocaleStringNoCase(item1->strLeecher, item2->strLeecher);
			break;
		case 1: 
			iResult = CompareUnsigned(item1->uiType, item2->uiType);
			break;
		case 2: 
			iResult = CompareLocaleStringNoCase(item1->strReason, item2->strReason);
			break;
		case 3: 
			iResult = item1->bBanned - item2->bBanned;
			break;
		case 4: 
			iResult = item1->bCase - item2->bCase;
			break;
		default:
			iResult = 0;
			break;
	}

	// SLUGFILLER: multiSort remove - handled in parent class
	return iResult;
}

void CAntiLeechListCtrl::OnNMDblclk(NMHDR *pNMHDR, LRESULT *pResult)
{
	*pResult = 0;
}

void CAntiLeechListCtrl::OnGetDispInfo(NMHDR *pNMHDR, LRESULT *pResult)
{
	NMLVDISPINFO *pDispInfo = reinterpret_cast<NMLVDISPINFO*>(pNMHDR);

	if (theApp.emuledlg->IsRunning())
	{
		// Although we have an owner drawn listview control we store the text for the primary item in the listview, to be
		// capable of quick searching those items via the keyboard. Because our listview items may change their contents,
		// we do this via a text callback function. The listview control will send us the LVN_DISPINFO notification if
		// it needs to know the contents of the primary item.
		//
		// But, the listview control sends this notification all the time, even if we do not search for an item. At least
		// this notification is only sent for the visible items and not for all items in the list. Though, because this
		// function is invoked *very* often, no *NOT* put any time consuming code here in.

		if (pDispInfo->item.mask & LVIF_TEXT)
		{
			const CLeecherData* pClient = reinterpret_cast<CLeecherData*>(pDispInfo->item.lParam);
			if (pClient != NULL)
			{
				switch (pDispInfo->item.iSubItem)
				{
					case 0:
						if (pDispInfo->item.cchTextMax > 0){
							_tcsncpy(pDispInfo->item.pszText, pClient->strLeecher, pDispInfo->item.cchTextMax);
							pDispInfo->item.pszText[pDispInfo->item.cchTextMax-1] = _T('\0');
						}
						break;
					default:
						// shouldn't happen
						pDispInfo->item.pszText[0] = _T('\0');
						break;
				}
			}
		}
	}
	*pResult = 0;
}

void CAntiLeechListCtrl::OnLvnGetInfoTip(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMLVGETINFOTIP pGetInfoTip = reinterpret_cast<LPNMLVGETINFOTIP>(pNMHDR);

	if (pGetInfoTip->iSubItem == 0)
	{
		LVHITTESTINFO hti = {0};
		::GetCursorPos(&hti.pt);
		ScreenToClient(&hti.pt);
		if (SubItemHitTest(&hti) == -1 || hti.iItem != pGetInfoTip->iItem || hti.iSubItem != 0){
			// don' show the default label tip for the main item, if the mouse is not over the main item
			if ((pGetInfoTip->dwFlags & LVGIT_UNFOLDED) == 0 && pGetInfoTip->cchTextMax > 0 && pGetInfoTip->pszText[0] != '\0')
				pGetInfoTip->pszText[0] = '\0';
			return;
		}

		const CLeecherData* leecher = (CLeecherData*)GetItemData(pGetInfoTip->iItem);
		if (leecher && pGetInfoTip->pszText && pGetInfoTip->cchTextMax > 0)
		{
			CString info; //TODO			
			_tcsncpy(pGetInfoTip->pszText, info, pGetInfoTip->cchTextMax);
			pGetInfoTip->pszText[pGetInfoTip->cchTextMax-1] = _T('\0');
		}
	}
	*pResult = 0;
}