//this file is part of eMule
//Copyright (C)2002-2005 Merkur ( devs@emule-project.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#include "stdafx.h"
#include "emule.h"
#include "emuledlg.h" // [ionix] - Auto Update AntiLeechlist
#include "PPgAntiLeech.h"
#include "AntiLeech.h"
#include "UserMsgs.h"
#include "otherfunctions.h"
#include "HttpDownloadDlg.h"
#include "extractfile.h"
#include "Log.h"
#include <zlib/zlib.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

#define VERLIST theAntiLeechClass.m_BadVerList
#define MODLIST theAntiLeechClass.m_BadModList
#define NAMELIST theAntiLeechClass.m_BadNameList

IMPLEMENT_DYNAMIC(CPPgAntiLeech, CDialog)

BEGIN_MESSAGE_MAP(CPPgAntiLeech, CResizableDialog)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_ANTILEECH_LIST, OnLvnItemchangedAntileechList)
	ON_BN_CLICKED(IDC_ANTILEECH_BAN, OnSettingsChange)
	ON_BN_CLICKED(IDC_ANTILEECH_CASE, OnSettingsChange)
	ON_EN_CHANGE(IDC_ANTILEECH_REASON, OnSettingsChange)
	ON_BN_CLICKED(IDC_ANTILEECH_UPDATE, OnUpdateButton)
	ON_BN_CLICKED(IDC_APPLY, OnApplyBtn)
	ON_WM_HELPINFO() //<<< [ionix] - ionixguide
	ON_WM_DESTROY()
END_MESSAGE_MAP()

CPPgAntiLeech::CPPgAntiLeech()
: CResizableDialog(CPPgAntiLeech::IDD)
{
	m_bChanged = false;
	bCreated = false;
}

CPPgAntiLeech::~CPPgAntiLeech()
{
	for(int i = 0; i < m_lAntiLeech.GetItemCount(); ++i)
		delete (CLeecherData*)m_lAntiLeech.GetItemData(i);
	m_lAntiLeech.DeleteAllItems();
}

void CPPgAntiLeech::DoDataExchange(CDataExchange* pDX)
{
	CResizableDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_ANTILEECH_LIST, m_lAntiLeech);
}

BOOL CPPgAntiLeech::OnInitDialog()
{
	CResizableDialog::OnInitDialog();
	InitWindowStyles(this);

	// TODO: Add extra initialization for CListCtrl here
	m_lAntiLeech.Init();

	AddAnchor(IDC_APPLY, BOTTOM_RIGHT);
	AddAnchor(IDOK, BOTTOM_RIGHT);
	AddAnchor(IDC_ANTILEECH_REPORT, BOTTOM_LEFT, BOTTOM_RIGHT);

	LoadSettings();
	Localize();

	// note: there are better classes to create a pure hyperlink, however since it is only needed here
	//		 we rather use an already existing class
	CRect rect;
	GetDlgItem(IDC_ANTILEECH_REPORT)->GetWindowRect(rect);
	::MapWindowPoints(NULL, m_hWnd, (LPPOINT)&rect, 2);
	m_wndContact.CreateEx(NULL,0,_T("MsgWnd"),WS_BORDER | WS_VISIBLE | WS_CHILD | HTC_WORDWRAP | HTC_UNDERLINE_HOVER,rect.left,rect.top,rect.Width(),rect.Height(),m_hWnd,0);
	m_wndContact.SetBkColor(::GetSysColor(COLOR_3DFACE)); // still not the right color, will fix this later (need to merge the .rc file before it changes ;) )
	m_wndContact.SetFont(GetFont());
	if (!bCreated)
	{
		bCreated = true;
		const CString URL = _T("mailto:leechophobia@airu.de");
		m_wndContact.AppendText(GetResString(IDS_FOUNDLEECHER_TEXT));
		m_wndContact.AppendHyperLink(GetResString(IDS_FOUNDLEECHER_LINK), URL, URL, 0, 0);
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CPPgAntiLeech::LoadSettings(void)
{
	for(POSITION pos = VERLIST.GetHeadPosition(); pos; VERLIST.GetNext(pos))
	{
		CBaninfo info = VERLIST.GetValueAt(pos);
		m_lAntiLeech.AddLeecher(new CLeecherData(VERLIST.GetKeyAt(pos), 2, info.GetReason(), info.Ban(), info.UseCase()));
	}

	for(POSITION pos = MODLIST.GetHeadPosition(); pos; MODLIST.GetNext(pos))
	{
		CBaninfo info = MODLIST.GetValueAt(pos);
		m_lAntiLeech.AddLeecher(new CLeecherData(MODLIST.GetKeyAt(pos), 1, info.GetReason(), info.Ban(), info.UseCase()));
	}

	for(POSITION pos = NAMELIST.GetHeadPosition(); pos; NAMELIST.GetNext(pos))
	{
		CBaninfo info = NAMELIST.GetValueAt(pos);
		m_lAntiLeech.AddLeecher(new CLeecherData(NAMELIST.GetKeyAt(pos), 0, info.GetReason(), info.Ban(), info.UseCase()));
	}
}

BOOL CPPgAntiLeech::OnApply()
{
	if(!IsModified())
		return FALSE;
	PreSetModified(0);

	POSITION pos = m_lAntiLeech.GetFirstSelectedItemPosition();
	while (pos)
	{
		const int i = m_lAntiLeech.GetNextSelectedItem(pos);
		CLeecherData* leecher = (CLeecherData*)m_lAntiLeech.GetItemData(i);
		leecher->bBanned = IsDlgButtonChecked(IDC_ANTILEECH_BAN)!=0;
		leecher->bCase = IsDlgButtonChecked(IDC_ANTILEECH_CASE)!=0;
		
		CString reason;
		GetDlgItem(IDC_ANTILEECH_REASON)->GetWindowText(reason);
		if(!reason.IsEmpty())
			leecher->strReason = reason;
		m_lAntiLeech.Update(i);
		CBaninfo info(leecher->strReason, leecher->bBanned, leecher->bCase);
		if(leecher->uiType == 1)
			MODLIST.SetAt(leecher->strLeecher, info);
		else if(leecher->uiType == 2)
			VERLIST.SetAt(leecher->strLeecher, info);
		else if(leecher->uiType == 0)
			NAMELIST.SetAt(leecher->strLeecher, info);
	}

	return TRUE;
}

BOOL CPPgAntiLeech::OnCommand(WPARAM wParam, LPARAM lParam)
{
	if (wParam == UM_ITEMSTATECHANGED)
		PreSetModified();
	else if (wParam == ID_HELP)
	{
		OnHelp();
		return TRUE;
	}
	return CResizableDialog::OnCommand(wParam, lParam);
}

void CPPgAntiLeech::Localize(void)
{
	if(m_hWnd)
	{
		SetWindowText(_T("AntiLeech"));
		SetDlgItemText(IDC_APPLY,_T("Apply"));
		SetDlgItemText(IDOK,GetResString(IDS_FD_CLOSE));
	}
}

void CPPgAntiLeech::OnDestroy()
{
	CResizableDialog::OnDestroy();
}

void CPPgAntiLeech::OnLvnItemchangedAntileechList(NMHDR *pNMHDR, LRESULT *pResult)
{
	//LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
	OnSettingsChange();
	*pResult = 0;
}

void CPPgAntiLeech::OnUpdateButton()
{
	UpdateAntiLeecherList();
}

#define DEFAULT_PATH(s)	(thePrefs.GetConfigDir()+s)

void CPPgAntiLeech::UpdateAntiLeecherList(const CString& m_sURL)
{
	CString sbuffer;
	CString strURL;
	if(m_sURL.IsEmpty())		
		strURL = _T("http://emuleionix.sourceforge.net/antileech.php?get=txt&both");
	else
	{
		strURL = m_sURL;
		strURL.TrimRight(_T(".dat"));
		strURL.TrimRight(_T(".zip"));
		strURL.TrimRight(_T(".txt"));
		strURL.Append(_T(".txt"));
	}

	TCHAR szTempFilePath[_MAX_PATH];
	_tmakepath(szTempFilePath, NULL, thePrefs.GetConfigDir(), _T("antileechver"), _T("tmp"));	

	CHttpDownloadDlg dlgDownload;
	dlgDownload.m_sUserAgent = _T("eMule ") + theApp.m_strCurVersionLong + _T(" (") + theApp.sysinfo->GetOSType() +  _T(")");
	dlgDownload.m_strTitle = _T("Downloading AntiLeech version file...");
	dlgDownload.m_sURLToDownload = strURL;
	dlgDownload.m_sFileToDownloadInto = szTempFilePath;

	if (dlgDownload.DoModal() != IDOK)
	{
		_tremove(szTempFilePath);
		LogError(LOG_STATUSBAR, _T("Error while downloading \"%s\""), strURL);
		return;
	}

	CStdioFile file;
	CString strVersion = _T("");
	CString msg = _T("CHANGELOG\n");
	if(file.Open(szTempFilePath, CFile::modeReadWrite | CFile::shareDenyWrite))
	{
		file.ReadString(strVersion);
		CString line;
		while(file.ReadString(line))
			msg.AppendFormat(_T("%s\n"), line);
		file.Close();		
		strVersion.Trim();
	}
	_tremove(szTempFilePath);
	msg.Append(_T("\nDownload complete - update now?"));

	if ((thePrefs.GetAntiLeechDatVersion() < (uint32)_tstoi(strVersion) 
		|| !PathFileExists(thePrefs.GetConfigDir()+_T("antileech.dat")))
		&& AfxMessageBox(msg, MB_YESNO) == IDYES)
	{
		if(m_sURL.IsEmpty())			
			strURL = _T("http://emuleionix.sourceforge.net/antileech.php?get=zip");
		else
			strURL = m_sURL;

		_tmakepath(szTempFilePath, NULL, thePrefs.GetAppDir(), _T("antileechdata"), _T("tmp"));

		CHttpDownloadDlg dlgDownload;
		dlgDownload.m_strTitle = _T("Downloading AntiLeech data file...");
		dlgDownload.m_sURLToDownload = strURL;
		dlgDownload.m_sFileToDownloadInto = szTempFilePath;
		if (dlgDownload.DoModal() != IDOK)
		{
			_tremove(szTempFilePath);
			LogError(_T("Error while downloading \"%s\""), strURL);
			return;
		}

		//Move all files to backup versions...
		const CString sigfile = _T("antileech.sig");
		const CString preffile = _T("antileech.pref");
		const CString datfile = _T("antileech.dat");
		const CString txtfile = _T("antileech.txt");
		_trename(DEFAULT_PATH(sigfile),	DEFAULT_PATH(sigfile+_T(".tmp")));
		_trename(DEFAULT_PATH(preffile),DEFAULT_PATH(preffile+_T(".tmp")));
		_trename(DEFAULT_PATH(datfile),	DEFAULT_PATH(datfile+_T(".tmp")));
		_trename(DEFAULT_PATH(txtfile),	DEFAULT_PATH(txtfile+_T(".tmp")));

		//Extract all files...
		try
		{
			bool bIsZipFile = false;
			bool bUnzipped = false;
			Unzip(szTempFilePath, thePrefs.GetConfigDir(), sigfile, bIsZipFile, bUnzipped);
			if(bIsZipFile && !bUnzipped)
			{
				LogError(LOG_STATUSBAR, _T("Error while extracting \"%s\""), sigfile);
				throw;
			}


			bIsZipFile = false;
			bUnzipped = false;
			Unzip(szTempFilePath, thePrefs.GetConfigDir(), preffile, bIsZipFile, bUnzipped);
			if(bIsZipFile && !bUnzipped)
			{
				LogError(LOG_STATUSBAR, _T("Error while extracting \"%s\""), preffile);
				throw;
			}

			bIsZipFile = false;
			bUnzipped = false;
			Unzip(szTempFilePath, thePrefs.GetConfigDir(), datfile, bIsZipFile, bUnzipped);
			if(bIsZipFile && !bUnzipped)
			{
				LogError(LOG_STATUSBAR, _T("Error while extracting \"%s\""), datfile);
				throw;
			}

			bIsZipFile = false;
			bUnzipped = false;
			Unzip(szTempFilePath, thePrefs.GetConfigDir(), txtfile, bIsZipFile, bUnzipped);			
			if(bIsZipFile && !bUnzipped)
			{
				LogError(LOG_STATUSBAR, _T("Error while extracting \"%s\""), txtfile);
				//throw; //this is not deadly ;)
			}

			thePrefs.SetAntiLeechDatVersion(_tstoi(strVersion));
			thePrefs.Save();
			theAntiLeechClass.LoadFromFile();
			theAntiLeechClass.CheckKeys();

			for(int i = 0; i < m_lAntiLeech.GetItemCount(); ++i)
				delete (CLeecherData*)m_lAntiLeech.GetItemData(i);
			m_lAntiLeech.DeleteAllItems();
			LoadSettings();

			_tremove(DEFAULT_PATH(sigfile+_T(".tmp")));
			_tremove(DEFAULT_PATH(preffile+_T(".tmp")));
			_tremove(DEFAULT_PATH(datfile+_T(".tmp")));
			_tremove(DEFAULT_PATH(txtfile+_T(".tmp")));
		}
		catch(...)
		{
			//Move all files back - there was something wrong!
			_trename(DEFAULT_PATH(sigfile+_T(".tmp")),	DEFAULT_PATH(sigfile));
			_trename(DEFAULT_PATH(preffile+_T(".tmp")),	DEFAULT_PATH(preffile));
			_trename(DEFAULT_PATH(datfile+_T(".tmp")),	DEFAULT_PATH(datfile));
			_trename(DEFAULT_PATH(txtfile+_T(".tmp")),	DEFAULT_PATH(txtfile));
		}
	}
}

BOOL CPPgAntiLeech::PreTranslateMessage(MSG* pMsg)
{
	return CResizableDialog::PreTranslateMessage(pMsg);
}

//>>> [ionix] - ionixguide
void CPPgAntiLeech::OnHelp()
{
	theApp.ShowIonixHelp();
}

BOOL CPPgAntiLeech::OnHelpInfo(HELPINFO* pHelpInfo)
{
	OnHelp();
	return TRUE;
}
//<<< [ionix] - ionixguide

//>>> [ionix] - Auto Update AntiLeechlist
/////////////////////////////////////////////////////////////////////////////////////////
///CAutoUpdateAL
IMPLEMENT_DYNCREATE(CAutoUpdateAL, CWinThread)

CAutoUpdateAL::CAutoUpdateAL()
{
}
CAutoUpdateAL::~CAutoUpdateAL()
{
}

BOOL CAutoUpdateAL::InitInstance()
{
	DbgSetThreadName("AutoUpdate AntiLeech");
	InitThreadLocale();
	return TRUE;
}

int CAutoUpdateAL::Run()
{
	// some users expired problems with the splashscreen. so we do this in an own thread and wait
	// until the splashscreen is deleted
	while(theApp.emuledlg->m_pSplashWnd != NULL)
		Sleep(1000);

	CPPgAntiLeech CAntiLeech;
	CAntiLeech.UpdateAntiLeecherList();
	return 0;
}
//<<< [ionix] - Auto Update AntiLeechlist