//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//////////////////////////////////////////////////////////////////////////
//Please note!
//
//This file contains code parts from several mods out there, namingly
//MAELLA, Phoenix and a tiny bit of Xtreme - ThX to their modders!
//Most of their comments however were either useless or confusing or in the 
//way so they got removed...
//
//////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "NAFC.h"
#include "Emule.h"
#include "Log.h"
#include "Preferences.h"
#include "opcodes.h"
#include "Statistics.h"
#include "BandwidthControl.h" //>>> LIMITER

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CNAFC::CNAFC()
:   m_statisticHistory(1024) // size  ~= 1024*(4*8+3*4) = 1024*44 bytes = 44 KBytes
{
	m_statistic.networkOut = 0;
	m_statistic.networkIn = 0;
	m_statistic.eMuleOutOverall = 0;
	m_statistic.eMuleInOverall = 0;
	m_statistic.timeStamp = ::GetTickCount();
	m_maxUploadLimit = !thePrefs.IsAutoMaxUpload() ? thePrefs.GetMaxBCUpload()*1024 : thePrefs.GetMaxGraphUploadRate()*1024;
	m_maxDownloadLimit = !thePrefs.IsAutoMaxDownload() ? thePrefs.GetMaxBCDownload()*1024 : thePrefs.GetMaxGraphDownloadRate()*1024;
	m_errorTraced = false;

	// Keep last result to detect an overflow
	//m_networkOut = 0;
	//m_networkIn = 0;

	// Dynamic load library iphlpapi.dll => user of win95
	m_hIphlpapi = ::LoadLibrary(_T("iphlpapi.dll"));
	if(m_hIphlpapi != NULL)
	{
		m_fGetIfTable = (GETIFTABLE)GetProcAddress(m_hIphlpapi, "GetIfTable");
		m_fGetIpAddrTable = (GETIPADDRTABLE)GetProcAddress(m_hIphlpapi, "GetIpAddrTable");
		m_fGetIfEntry = (GETIFENTRY)GetProcAddress(m_hIphlpapi, "GetIfEntry");
		m_fGetNumberOfInterfaces = (GETNUMBEROFINTERFACES)GetProcAddress(m_hIphlpapi, "GetNumberOfInterfaces");

		theApp.QueueDebugLogLine(false, _T("NAFC: Succeeded to load library iphlpapi.dll"));
	}
	else 
	{
		m_fGetIfTable = NULL;
		m_fGetIpAddrTable = NULL;
		m_fGetIfEntry = NULL;
		m_fGetNumberOfInterfaces = NULL;

		theApp.QueueDebugLogLine(false, _T("NAFC: Failed to load library iphlpapi.dll"));
	}

	//Xman new adapter selection
	wasNAFCLastActive = thePrefs.GetNAFCEnable();

	// Cache index value
	m_currentAdapterIndex = thePrefs.GetNAFCSelection(); 
	if (m_currentAdapterIndex != 0 && !CheckAdapterIndex(NULL, true))
		m_currentAdapterIndex = 0;
}

CNAFC::~CNAFC()
{
	// Unload library
	if(m_hIphlpapi != NULL)
		::FreeLibrary(m_hIphlpapi);
}

//Xman new adapter selection
bool CNAFC::CheckAdapterIndex(const uint32& highid, const bool& checkonlyexisting)
{
	// Check if the library was successfully loaded
	if(m_fGetNumberOfInterfaces != NULL && m_fGetIfTable != NULL && m_fGetIpAddrTable != NULL)
	{
		DWORD dwNumIf = 0;
		if(m_fGetNumberOfInterfaces(&dwNumIf) == NO_ERROR && dwNumIf > 0 )
		{
			BYTE buffer[10*sizeof(MIB_IFROW)];
			ULONG size = sizeof(buffer);
			MIB_IFTABLE& mibIfTable = reinterpret_cast<MIB_IFTABLE&>(buffer[0]);
			if(m_fGetIfTable(&mibIfTable, &size, true) == NO_ERROR)
			{
				// Trace list of Adapters
				if(!m_errorTraced)
				{
					for(DWORD dwNumEntries = 0; dwNumEntries < mibIfTable.dwNumEntries; dwNumEntries++)
					{
						const MIB_IFROW& mibIfRow = mibIfTable.table[dwNumEntries];
						theApp.QueueDebugLogLine(false, _T("NAFC: Adapter %u is '%s'"), mibIfRow.dwIndex, (CString)mibIfRow.bDescr);
					}
				}

				// Retrieve the default used IP (=> in case of multiple adapters)
				char hostName[256];
				if(gethostname(hostName, sizeof(hostName)) == 0)
				{
					hostent* lphost = gethostbyname(hostName);
					if(lphost != NULL)
					{
						DWORD dwAddr = ((LPIN_ADDR)lphost->h_addr)->s_addr;
						// Pick the interface matching the IP
						BYTE buffer[10*sizeof(MIB_IPADDRROW)];
						ULONG size = sizeof(buffer);
						MIB_IPADDRTABLE& mibIPAddrTable = reinterpret_cast<MIB_IPADDRTABLE&>(buffer[0]);
						if(m_fGetIpAddrTable(&mibIPAddrTable, &size, FALSE) == 0)
						{
							//Xman: first we seek the highid from the server
							for(DWORD i = 0; i < mibIPAddrTable.dwNumEntries; i++)
							{
								if(!checkonlyexisting)
								{
								if(mibIPAddrTable.table[i].dwAddr == highid)
								{
									//const MIB_IPADDRROW& row = mibIPAddrTable.table[i];
									m_errorTraced = false;
									theApp.QueueDebugLogLine(false, _T("NAFC found by IP: Selected adapter with index %u"), mibIPAddrTable.table[i].dwIndex);
									//reactivate NAFC	
									if(wasNAFCLastActive)
										thePrefs.SetNAFCEnable(true);
										SelectAdapter(mibIPAddrTable.table[i].dwIndex);
									return true;
								}
							}
								else
								{
									if(m_currentAdapterIndex == mibIPAddrTable.table[i].dwIndex)
									{
										m_errorTraced = false;
										theApp.QueueDebugLogLine(false, _T("NAFC verified: Selected adapter with index %u"), mibIPAddrTable.table[i].dwIndex);
										//reactivate NAFC	
										if(wasNAFCLastActive)
											thePrefs.SetNAFCEnable(true);
										SelectAdapter(m_currentAdapterIndex);
										return true;
									}
								}
							}
							//Xman: if highid not found, search the hostip
							if(!checkonlyexisting)
							{
							for(DWORD i = 0; i < mibIPAddrTable.dwNumEntries; i++)
							{
								if(mibIPAddrTable.table[i].dwAddr == dwAddr)
								{
									//const MIB_IPADDRROW& row = mibIPAddrTable.table[i];
									m_errorTraced = false;
										theApp.QueueDebugLogLine(false, _T("NAFC: Selected adapter with index %u"), mibIPAddrTable.table[i].dwIndex);
									//reactivate NAFC	
									if(wasNAFCLastActive)
										thePrefs.SetNAFCEnable(true);
										SelectAdapter(mibIPAddrTable.table[i].dwIndex);
									return true;
								}
							}
						}
						}
						else 
						{
							if(!m_errorTraced)
							{
								m_errorTraced = true;
								theApp.QueueDebugLogLine(false, _T("NAFC: Failed to get IP tables error=0x%x"), ::GetLastError());
							}
							return false;						}
					}
				}
			}
			else 
			{
				if(!m_errorTraced)
				{
					m_errorTraced = true;
					theApp.QueueDebugLogLine(false, _T("NAFC: Failed to get tables of interface error=0x%x"), ::GetLastError());
				}
				return false;
			}
		}
		else 
		{
			if(!m_errorTraced)
			{
				m_errorTraced = true;
				theApp.QueueDebugLogLine(false, _T("NAFC: Failed to get the number of interface error=0x%x"), ::GetLastError());
			}
			return false;
		}
	}
	if(!m_errorTraced)
	{
		m_errorTraced = true;
		theApp.QueueDebugLogLine(false, _T("NAFC: Failed to verify selected adapter with index %u"), m_currentAdapterIndex);
	}	
	return false;
}
//Xman new adapter selection end

/*DWORD CNAFC::GetAdapterIndex()
{
	// Check if the library was successfully loaded
	if(m_fGetNumberOfInterfaces != NULL && m_fGetIfTable != NULL && m_fGetIpAddrTable != NULL)
	{
		DWORD dwNumIf = 0;
		if(m_fGetNumberOfInterfaces(&dwNumIf) == NO_ERROR && dwNumIf > 0 )
		{
			BYTE buffer[10*sizeof(MIB_IFROW)];
			ULONG size = sizeof(buffer);
			MIB_IFTABLE& mibIfTable = reinterpret_cast<MIB_IFTABLE&>(buffer[0]);
			if(m_fGetIfTable(&mibIfTable, &size, true) == NO_ERROR)
			{
				// Trace list of Adapters
				if(!m_errorTraced)
				{
					for(DWORD dwNumEntries = 0; dwNumEntries < mibIfTable.dwNumEntries; dwNumEntries++)
					{
						const MIB_IFROW& mibIfRow = mibIfTable.table[dwNumEntries];
						theApp.QueueDebugLogLine(false, _T("NAFC: Adapter %u is '%s'"), mibIfRow.dwIndex, (CString)mibIfRow.bDescr);
					}
				}

				// Retrieve the default used IP (=> in case of multiple adapters)
				char hostName[256];
				if(gethostname(hostName, sizeof(hostName)) == 0)
				{
					hostent* lphost = gethostbyname(hostName);
					if(lphost != NULL)
					{
						DWORD dwAddr = ((LPIN_ADDR)lphost->h_addr)->s_addr;
						// Pick the interface matching the IP
						BYTE buffer[10*sizeof(MIB_IPADDRROW)];
						ULONG size = sizeof(buffer);
						MIB_IPADDRTABLE& mibIPAddrTable = reinterpret_cast<MIB_IPADDRTABLE&>(buffer[0]);
						if(m_fGetIpAddrTable(&mibIPAddrTable, &size, FALSE) == 0)
						{
							for(DWORD i = 0; i < mibIPAddrTable.dwNumEntries; i++)
							{
								if(mibIPAddrTable.table[i].dwAddr == dwAddr)
								{
									//const MIB_IPADDRROW& row = mibIPAddrTable.table[i];

									m_errorTraced = false;
									theApp.QueueDebugLogLine(false, _T("NAFC: Select adapter with index %u"), mibIPAddrTable.table[i].dwIndex);
									return mibIPAddrTable.table[i].dwIndex;
								}
							}
						}
						else 
						{
							if(!m_errorTraced)
							{
								m_errorTraced = true;
								theApp.QueueDebugLogLine(false, _T("NAFC: Failed to get IP tables error=0x%x"), ::GetLastError());
								return 0;
							}
						}
					}
				}
			}
			if(!m_errorTraced)
			{
				m_errorTraced = true;
				theApp.QueueDebugLogLine(false, _T("NAFC: Failed to get tables of interface error=0x%x"), ::GetLastError());
				return 0;
			}
		}
		if(!m_errorTraced)
		{
			m_errorTraced = true;
			theApp.QueueDebugLogLine(false, _T("NAFC: Failed to get the number of interface error=0x%x"), ::GetLastError());
			return 0;
		}
	}
	return 0;
}*/

//                 Only the code related to the instance bandwidth is 
//                 multithread-safe (see tag )
//
void CNAFC::Process()
{
	static DWORD processtime;
	if(::GetTickCount()-processtime >= 1000)
	{
		processtime = ::GetTickCount();

		// Try to get the Adapter Index to access to the right interface
		if(thePrefs.GetNAFCEnable() && m_currentAdapterIndex == 0)
		{
			//m_currentAdapterIndex = GetAdapterIndex();			

			// Disable NAFC
			//if(m_currentAdapterIndex == 0)
			//	thePrefs.SetNAFCEnabled(false);
			//SelectAdapter(m_currentAdapterIndex);

			CheckAdapterIndex(NULL, false);
		}

		m_statisticLocker.Lock();
		
		// Update the datarate directly from the network Adapter
		if(m_currentAdapterIndex != 0 && m_fGetIfEntry != NULL)
		{
			static int s_Log; // Static initiate with zero
			MIB_IFROW ifRow;       
			ifRow.dwIndex = m_currentAdapterIndex;
			if(m_fGetIfEntry(&ifRow) == NO_ERROR)
			{
				s_Log = 0;

				//Xman prevent overflow on adapterchange:
				// Add the delta, since the last measure (convert 32 to 64 bits)
				//if(ifRow.dwOutOctets >= m_networkOut)
				//	m_statistic.networkOut += (DWORD)(ifRow.dwOutOctets - m_networkOut);
				//if(ifRow.dwInOctets >= m_networkIn)					
				//	m_statistic.networkIn += (DWORD)(ifRow.dwInOctets - m_networkIn);

				// Keep last measure
				//m_networkOut = ifRow.dwOutOctets; 
				//m_networkIn = ifRow.dwInOctets;

				m_statistic.networkOut = ifRow.dwOutOctets;
				m_statistic.networkIn = ifRow.dwInOctets;
			}
			else 
			{
				if(s_Log == 0)
				{
					s_Log = 1;
					theApp.QueueDebugLogLine(false, _T("NAFC: Failed to retrieve adapter traffic, error=0x%x"), ::GetLastError());
					wasNAFCLastActive = thePrefs.GetNAFCEnable(); //Xman  
					// Disable NAFC
					thePrefs.SetNAFCEnable(false);
				}
			}
		}

//WiZaRd: that's dirty, I know - though, I don't care :P
		m_statistic.eMuleOutOverall = theStats.sessionSentBytes;
		if(thePrefs.IsIncludeOverhead())
			m_statistic.eMuleOutOverall += theStats.GetUpDataOverheadFileRequest() + 
				theStats.GetUpDataOverheadSourceExchange() + 
				theStats.GetUpDataOverheadServer() + 
				theStats.GetUpDataOverheadKad() + 
				theStats.GetUpDataOverheadOther();

		m_statistic.eMuleInOverall = theStats.sessionReceivedBytes;
		if(thePrefs.IsIncludeOverhead())
			m_statistic.eMuleInOverall += theStats.GetDownDataOverheadFileRequest() + 
				theStats.GetDownDataOverheadSourceExchange() + 
				theStats.GetDownDataOverheadServer() + 
				theStats.GetDownDataOverheadKad() + 
				theStats.GetDownDataOverheadOther();

		// Update large history list for the history graph
		m_statisticHistory.AddHead(Statistic(m_statistic, ::GetTickCount()));

		// Trunk size of the list (The timestamp is more accurate than the period of Process())
		const uint32 averageMinTime = (uint32)(60000 * thePrefs.GetStatsAverageMinutes());
#pragma warning(disable:4127)
		while(true)
#pragma warning(default:4127)
		{
			const uint32 deltaTime = m_statisticHistory.GetHead().timeStamp - m_statisticHistory.GetTail().timeStamp;
			if(deltaTime <= averageMinTime)
				break; // exit loop
			m_statisticHistory.RemoveTail(); // Trunk size
		}
		m_statisticLocker.Unlock();

//--> Calculate the dynamic limits
		float tmp_maxUploadLimit = thePrefs.GetMaxGraphUploadRate()*1024; //in [Bytes/s]
		float tmp_maxDownloadLimit = thePrefs.GetMaxGraphDownloadRate()*1024; //in [Bytes/s]
		if(thePrefs.GetNAFCEnable())
		{    
			// Remark: Because there is only ONE writer thread to m_statisticHistory, there is no need to 
			//         protect its access inside this method. 
			//         => m_statisticHistory is only modified inside this method
			//         => Its access in GetDatarates() + GetFullHistoryDatarates() still must be protected
			if(m_statisticHistory.GetSize() > 1)
			{
				const Statistic& newestSample = m_statisticHistory.GetHead();
				const Statistic& oldestSample = m_statisticHistory.GetAt(m_statisticHistory.FindIndex(1));
				const uint32 deltaTime = (newestSample.timeStamp - oldestSample.timeStamp); // in [ms]
				if (deltaTime == 0) 
					return; //just to be sure...
				const uint64 eMuleOutOverall = (1000 * (uint32)(newestSample.eMuleOutOverall - oldestSample.eMuleOutOverall) / deltaTime); // in [Bytes/s]
				const uint64 eMuleInOverall = (1000 * (uint32)(newestSample.eMuleInOverall - oldestSample.eMuleInOverall) / deltaTime); // in [Bytes/s]
				const uint64 networkOut = (1000 * (uint32)(newestSample.networkOut - oldestSample.networkOut) / deltaTime); // in [Bytes/s]
				const uint64 networkIn = (1000 * (uint32)(newestSample.networkIn - oldestSample.networkIn) / deltaTime); // in [Bytes/s]

//>>> Auto-Limit-Detection
/*
				if(thePrefs.GetNAFCAutoLimit())
				{
					if(networkOut > tmp_maxUploadLimit)
					{
						theApp.QueueDebugLogLine(false, _T("Network send detected %u sent bytes but limit was set to %u bytes - corrected!"), networkOut, (uint32)tmp_maxUploadLimit);
						tmp_maxUploadLimit = networkOut;
						thePrefs.SetMaxGraphUploadRate(networkOut/1024.0f);
					}
					if(networkIn > tmp_maxDownloadLimit)
					{
						theApp.QueueDebugLogLine(false, _T("Network rec detected %u received bytes but limit was set to %u bytes - corrected!"), networkIn, (uint32)tmp_maxDownloadLimit);
						tmp_maxDownloadLimit = networkIn;
						thePrefs.SetMaxGraphDownloadRate(networkIn/1024.0f);
					}
				}
*/
//<<< Auto-Limit-Detection

				//theApp.QueueDebugLogLine(false, _T("MaxUp: %.1f - Network: %.1f - eMule: %.1f - Speed: %.1f"), 
				//	tmp_maxUploadLimit/1024.0f, networkOut/1024.0f, eMuleOutOverall/1024.0f,
				//	(tmp_maxUploadLimit - networkOut + eMuleOutOverall)/1024.0f
				//	);

				tmp_maxUploadLimit = (tmp_maxUploadLimit //max
					- networkOut			//all sent
					+ eMuleOutOverall);		//sent by eMule						

				if(tmp_maxUploadLimit < 0)
				{
					//ASSERT(0); //happens pretty often if using caching...
					tmp_maxUploadLimit = 0;
				}

				//theApp.QueueDebugLogLine(false, _T("MaxDL: %.1f - Network: %.1f - eMule: %.1f - Speed: %.1f"), 
				//	tmp_maxDownloadLimit/1024.0f, networkIn/1024.0f, eMuleInOverall/1024.0f,
				//	(tmp_maxDownloadLimit - networkIn + eMuleInOverall)/1024.0f
				//	);

				tmp_maxDownloadLimit = (tmp_maxDownloadLimit //max
					- networkIn				//all received
					+ eMuleInOverall);		//received by eMule
				if(tmp_maxDownloadLimit < 0)
				{
					//ASSERT(0); //happens pretty often if using caching...
					tmp_maxDownloadLimit = 0;
				}
			}
		}

		//Full control disobeys user limits and only uses adapter feedback
		if(thePrefs.GetNAFCFullControl())
		{
			m_maxUploadLimit = tmp_maxUploadLimit;
			m_maxDownloadLimit = tmp_maxDownloadLimit;
		}
		else
		{
			m_maxUploadLimit = max(tmp_maxUploadLimit, thePrefs.GetMinBCUpload()*1024);
			if(!thePrefs.IsAutoMaxUpload())
				m_maxUploadLimit = min(tmp_maxUploadLimit, thePrefs.GetMaxBCUpload()*1024);

			m_maxDownloadLimit = max(tmp_maxDownloadLimit, thePrefs.GetMinBCDownload()*1024);
			if(!thePrefs.IsAutoMaxDownload())
				m_maxDownloadLimit = min(tmp_maxDownloadLimit, thePrefs.GetMaxBCDownload()*1024);
		}

		m_maxUploadLimit /= 1024.0f; //in kB/s
		m_maxDownloadLimit /= 1024.0f; //in kB/s
//>>> LIMITERS
		m_maxDownloadLimit = theApp.BC->GetMaxRatioDownload(m_maxUploadLimit, m_maxDownloadLimit);
//<<< LIMITERS
//<-- Calculate the dynamic limits
	}
	else if(thePrefs.GetNAFCEnable())
	{
		// Retrieve the network flow => necessary for the NAFC with 'Auto U/D limit'
		if(m_currentAdapterIndex != 0 && m_fGetIfEntry != NULL)
		{
			MIB_IFROW ifRow;       
			ifRow.dwIndex = m_currentAdapterIndex;
			if(m_fGetIfEntry(&ifRow) == NO_ERROR)
			{
				//m_statisticLocker.Lock();  
				//Xman prevent overflow on adapter change:
				// Add the delta, since the last measure (convert 32 to 64 bits)
				//if(ifRow.dwOutOctets >= m_networkOut)
				//	m_statistic.networkOut += (DWORD)(ifRow.dwOutOctets - m_networkOut);
				//if(ifRow.dwInOctets >= m_networkIn)					
				//	m_statistic.networkIn += (DWORD)(ifRow.dwInOctets - m_networkIn);

				//m_statisticLocker.Unlock();

				// Keep last measure
				//m_networkOut = ifRow.dwOutOctets;
				//m_networkIn = ifRow.dwInOctets;

				m_statistic.networkOut = ifRow.dwOutOctets;
				m_statistic.networkIn = ifRow.dwInOctets;
			}
		}
	}
}

//>>> NAFC Selection
void CNAFC::SelectNAFC()
{
	if(m_fGetNumberOfInterfaces != NULL && m_fGetIfTable != NULL && m_fGetIpAddrTable != NULL)
	{
		DWORD dwNumIf = 0;
		CString	selectedDescription = _T("");
		DWORD selectedIndex = m_currentAdapterIndex;
		uint32 selectedMTU = thePrefs.GetMSS();
		if(m_fGetNumberOfInterfaces(&dwNumIf) == NO_ERROR && dwNumIf > 0 )
		{
			BYTE buffer[10*sizeof(MIB_IFROW)];
			ULONG size = sizeof(buffer);
			MIB_IFTABLE& mibIfTable = reinterpret_cast<MIB_IFTABLE&>(buffer[0]);
			if(m_fGetIfTable(&mibIfTable, &size, true) == NO_ERROR)
			{
				// Trace list of Adapters
				//if(!m_errorTraced)
				{		
					CSelNAFCDlg* getNAFCDlg = new CSelNAFCDlg(theApp.emuledlg);                  
					BYTE buffer2[10*sizeof(MIB_IPADDRROW)];
					ULONG size2 = sizeof(buffer2);
					MIB_IPADDRTABLE& mibIPAddrTable = reinterpret_cast<MIB_IPADDRTABLE&>(buffer2[0]);
					NafcNumber = mibIfTable.dwNumEntries;
					for(DWORD dwNumEntries = 0; dwNumEntries < mibIfTable.dwNumEntries; dwNumEntries++)
					{
						const MIB_IFROW& mibIfRow = mibIfTable.table[dwNumEntries];
						m_NAFCInfo[dwNumEntries].description = (CString)mibIfRow.bDescr;
						m_NAFCInfo[dwNumEntries].index = mibIfRow.dwIndex;
						m_NAFCInfo[dwNumEntries].type = mibIfRow.dwType;
						m_NAFCInfo[dwNumEntries].mtu = mibIfRow.dwMtu;
						m_NAFCInfo[dwNumEntries].ip = 0;
						if(m_fGetIpAddrTable(&mibIPAddrTable, &size2, FALSE) == 0)
						{
							for(DWORD i = 0; i < mibIPAddrTable.dwNumEntries; i++)
							{
								if (mibIPAddrTable.table[i].dwIndex == mibIfRow.dwIndex)
									m_NAFCInfo[dwNumEntries].ip = mibIPAddrTable.table[i].dwAddr;
							}
						}	
					}
					getNAFCDlg->DoModal();
					int selection = getNAFCDlg->selection;					
					if(selection > -1 && selection < 9)
					{
						selectedIndex = m_NAFCInfo[selection].index;
						selectedDescription = m_NAFCInfo[selection].description;
						selectedMTU = m_NAFCInfo[selection].mtu;
					}
					delete getNAFCDlg;
				}
			}			
		}

		if(selectedIndex != 0 && selectedIndex != m_currentAdapterIndex)
		{
			// ionix only uses MSS that is MTU - 40 :D
			//thePrefs.m_iMaxFragSize = selectedMTU;
			thePrefs.SetMSS(selectedMTU-40);
			theApp.QueueDebugLogLine(false, _T("NAFC: New NAFC: %s with index %u - MTU set to %u"), selectedDescription, selectedIndex, selectedMTU);
			SelectAdapter(selectedIndex);
		}
	}
}
//<<< NAFC Selection

void CNAFC::SelectAdapter(const DWORD& adapter)
{
	if(adapter == m_currentAdapterIndex)
		return;

	m_currentAdapterIndex = adapter;
	thePrefs.SetNAFCSelection(adapter);

	m_statisticLocker.Lock();

	m_statisticHistory.RemoveAll();

	m_statistic.networkOut = 0;
	m_statistic.networkIn = 0;
	if(m_currentAdapterIndex != 0 && m_fGetIfEntry != NULL)
	{
		MIB_IFROW ifRow;       
		ifRow.dwIndex = m_currentAdapterIndex;
		if(m_fGetIfEntry(&ifRow) == NO_ERROR)
		{
			m_statistic.networkOut = ifRow.dwOutOctets;
			m_statistic.networkIn = ifRow.dwInOctets;
		}
	}

	m_statistic.eMuleOutOverall = theStats.sessionSentBytes;
	if(thePrefs.IsConnectionsOverHead())
		m_statistic.eMuleOutOverall += theStats.GetUpDataOverheadFileRequest() + 
			theStats.GetUpDataOverheadSourceExchange() + 
			theStats.GetUpDataOverheadServer() + 
			theStats.GetUpDataOverheadKad() + 
			theStats.GetUpDataOverheadOther();

	m_statistic.eMuleInOverall = theStats.sessionReceivedBytes;
	if(thePrefs.IsConnectionsOverHead())
		m_statistic.eMuleInOverall += theStats.GetDownDataOverheadFileRequest() + 
			theStats.GetDownDataOverheadSourceExchange() + 
			theStats.GetDownDataOverheadServer() + 
			theStats.GetDownDataOverheadKad() + 
			theStats.GetDownDataOverheadOther();

	m_statistic.timeStamp = ::GetTickCount();

	m_statisticHistory.AddHead(Statistic(m_statistic, ::GetTickCount()));

	m_statisticLocker.Unlock();
}