//this file is part of eMule
//Copyright (C)2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#include "stdafx.h"
#include "emule.h"
#include "UploadQueue.h"
#include "Packets.h"
#include "KnownFile.h"
#include "ListenSocket.h"
#include "Exceptions.h"
#include "Scheduler.h"
#include "PerfLog.h"
#include "UploadBandwidthThrottler.h"
#include "ClientList.h"
#include "LastCommonRouteFinder.h"
#include "DownloadQueue.h"
#include "FriendList.h"
#include "Statistics.h"
#include "MMServer.h"
#include "OtherFunctions.h"
#include "UpDownClient.h"
#include "SharedFileList.h"
#include "KnownFileList.h"
#include "Sockets.h"
#include "ClientCredits.h"
#include "Server.h"
#include "ServerList.h"
#include "WebServer.h"
#include "emuledlg.h"
#include "ServerWnd.h"
#include "TransferWnd.h"
#include "SearchDlg.h"
#include "StatisticsDlg.h"
#include "Kademlia/Kademlia/Kademlia.h"
#include "Kademlia/Kademlia/Prefs.h"
#include "PreferencesDlg.h" // Pawcio: BC
#include "safefile.h" // Pawcio: PowerShare
#include "partfile.h" // Pawcio: PowerShare
#include "Log.h"
#include "collection.h"
#include "./NAFC/NAFC.h" //>>> WiZaRd::NAFC

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

static uint32 m_uiFirewalledTimer = NULL; //>>> WiZaRd::Recheck Firewalled
static uint32 counter, sec, statsave;
static UINT _uSaveStatistics = 0;
// -khaos--+++> Added iupdateconnstats...
static uint32 igraph, istats, iupdateconnstats;
// <-----khaos-

//TODO rewrite the whole networkcode, use overlapped sockets.. sure....

CUploadQueue::CUploadQueue()
{
	m_uiFirewalledTimer = ::GetTickCount()+MIN2MS(2); //>>> WiZaRd::Recheck Firewalled
	VERIFY( (h_timer = SetTimer(0,0,100,UploadTimer)) != NULL );
	if (thePrefs.GetVerbose() && !h_timer)
		AddDebugLogLine(true,_T("Failed to create 'upload queue' timer - %s"),GetErrorMessage(GetLastError()));
	datarate = 0;
	counter=0;
	successfullupcount = 0;
	failedupcount = 0;
	totaluploadtime = 0;
	m_nLastStartUpload = 0;
	statsave=0;
	// -khaos--+++>
	iupdateconnstats=0;
	// <-----khaos-
	m_dwRemovedClientByScore = ::GetTickCount();
    m_iHighestNumberOfFullyActivatedSlotsSinceLastCall = 0;
    m_MaxActiveClients = 0;
    m_MaxActiveClientsShortTime = 0;

    m_lastCalculatedDataRateTick = 0;
    m_avarage_dr_sum = 0;
    friendDatarate = 0;

    m_dwLastResortedUploadSlots = 0;
	// Pawcio: BC
	m_nLastAddToReserved=::GetTickCount();
	MaxUpload=0;
	bandwidthUsed=false;
    activeSlots=0;
	endingTransfer=0;
	clientsrdy=0;
	reservedclientsrdy=0;
	// <--- Pawcio: BC
}

/**
 * Find the highest ranking client in the waiting queue, and return it.
 *
 * Low id client are ranked as lowest possible, unless they are currently connected.
 * A low id client that is not connected, but would have been ranked highest if it
 * had been connected, gets a flag set. This flag means that the client should be
 * allowed to get an upload slot immediately once it connects.
 *
 * @return address of the highest ranking client.
 */
CUpDownClient* CUploadQueue::FindBestClientInQueue(bool allowLowIdAddNextConnectToBeSet) 
{
	POSITION toadd = 0;	
	POSITION toaddlow = 0;
	uint32	bestscore = 0;
	uint32  bestlowscore = 0;

//>>> WiZaRd::PowerShare
	POSITION friendpos = 0;
	POSITION toaddPS = 0;	
	POSITION toaddlowPS = 0;
	uint32	bestscorePS = 0;
	uint32  bestlowscorePS = 0;
//<<< WiZaRd::PowerShare

	POSITION pos1, pos2;
	for (pos1 = waitinglist.GetHeadPosition();( pos2 = pos1 ) != NULL;)
	{
		waitinglist.GetNext(pos1);
		CUpDownClient* cur_client =	waitinglist.GetAt(pos2);
		//While we are going through this list.. Lets check if a client appears to have left the network..
		ASSERT ( cur_client->GetLastUpRequest() );
		if ((::GetTickCount() - cur_client->GetLastUpRequest() > MAX_PURGEQUEUETIME) || !theApp.sharedfiles->GetFileByID(cur_client->GetUploadFileID()) ||
			cur_client->NoPartsForClient()) // Pawcio: BC
		{
			//This client has either not been seen in a long time, or we no longer share the file he wanted anymore..
			cur_client->ClearWaitStartTime();
			RemoveFromWaitingQueue(pos2,true);
			continue;
		}

//>>> WiZaRd::PowerShare
		if(cur_client->IsFriend() 
			&& cur_client->GetFriendSlot()
			&& !cur_client->HasLowID())
			friendpos = pos2;
		if(friendpos)
			continue; //no need to go on...
//<<< WiZaRd::PowerShare

		// finished clearing
		uint32 cur_score = cur_client->GetScore(false);

//>>> WiZaRd::PowerShare
		if (cur_client->IsPowerShared())
		{
			// cur_client is more worthy than current best client that is ready to go (connected).
			if ( cur_score > bestscorePS)
			{
				if(!cur_client->HasLowID() || (cur_client->socket && cur_client->socket->IsConnected())) 
				{
					// this client is a HighID or a lowID client that is ready to go (connected)
					// and it is more worthy
					bestscorePS = cur_score;
					toaddPS = pos2;
				} 
				else if(allowLowIdAddNextConnectToBeSet && !cur_client->m_bAddNextConnect) 
				{
					// this client is a lowID client that is not ready to go (not connected)

					// now that we know this client is not ready to go, compare it to the best not ready client
					// the best not ready client may be better than the best ready client, so we need to check
					// against that client
					if (cur_score > bestlowscorePS)
					{
						// it is more worthy, keep it
						bestlowscorePS = cur_score;
						toaddlowPS = pos2;
					}
				}
			}
		}
		else 
		{
//<<< WiZaRd::PowerShare
			// cur_client is more worthy than current best client that is ready to go (connected).
			if ( cur_score > bestscore)
			{
				if(!cur_client->HasLowID() || (cur_client->socket && cur_client->socket->IsConnected())) 
				{
					// this client is a HighID or a lowID client that is ready to go (connected)
					// and it is more worthy
					bestscore = cur_score;
					toadd = pos2;
					// newclient = waitinglist.GetAt(toadd); // Pawcio: BC
				} 
				else if(allowLowIdAddNextConnectToBeSet && !cur_client->m_bAddNextConnect) 
				{
					// this client is a lowID client that is not ready to go (not connected)

					// now that we know this client is not ready to go, compare it to the best not ready client
					// the best not ready client may be better than the best ready client, so we need to check
					// against that client
					if (cur_score > bestlowscore)
					{
						// it is more worthy, keep it
						bestlowscore = cur_score;
						toaddlow = pos2;
						// lowclient = waitinglist.GetAt(toaddlow); // Pawcio: BC
					}
				}
			}
		} //<<< WiZaRd::PowerShare
	} //for all clients in queue

//>>> WiZaRd::PowerShare
	if (bestlowscorePS > bestscorePS && toaddlowPS && allowLowIdAddNextConnectToBeSet)
		waitinglist.GetAt(toaddlowPS)->m_bAddNextConnect = true;
	else if (bestlowscore > bestscore && toaddlow && allowLowIdAddNextConnectToBeSet)
		waitinglist.GetAt(toaddlow)->m_bAddNextConnect = true;
	
	if (!friendpos && !toaddPS && !toadd)
		return NULL;

	if(friendpos)
		return waitinglist.GetAt(friendpos);
	if(toaddPS)
		return waitinglist.GetAt(toaddPS);
	return waitinglist.GetAt(toadd);
//<<< WiZaRd::PowerShare
}

void CUploadQueue::InsertInUploadingList(CUpDownClient* newclient, bool toReserved) 
{
	//Lets make sure any client that is added to the list has this flag reset!
	newclient->m_bAddNextConnect = false;

    // Add it last
    theApp.uploadBandwidthThrottler->AddToStandardList(newclient->GetFileUploadSocket(), toReserved);
	// Pawcio: BC
	if (toReserved)
		reserveduploadinglist.AddTail(newclient);
	else
		uploadinglist.AddTail(newclient);
	/*if (toReserved)
		newclient->SetSlotNumber(uploadinglist.GetCount() + reserveduploadinglist.GetCount());
	else
		newclient->SetSlotNumber(uploadinglist.GetCount());*/
	// <--- Pawcio: BC
}

bool CUploadQueue::AddUpNextClient(LPCTSTR pszReason, CUpDownClient* directadd, bool toReserve){ // Pawcio: BC
	CUpDownClient* newclient = NULL;
	// select next client or use given client
	// Pawcio: BC
	if (!directadd && !toReserve && reserveduploadinglist.GetCount()){
		theApp.uploadBandwidthThrottler->MoveFromReserved();
		m_nLastStartUpload = ::GetTickCount();
		return true;
	}
	if (!toReserve && thePrefs.IsMinimizeOpenedSlots() /*&& MaxUpload != UNLIMITED*/) // [ionix] - changed
		return false;
	// <--- Pawcio:BC
	if (!directadd)
	{
        newclient = FindBestClientInQueue(true);
	}
	else 
	{
		newclient = directadd;
		/*if (!IsDownloading(newclient)){
            if (newclient->HasLowID())
                AddLogLine(true,"DirectAdd:  LowID: %s", newclient->GetUserName());
			else
                AddLogLine(true,"DirectAdd:  HighID: %s", newclient->GetUserName());
		}*/
	}

	if(newclient) 
	{
		//Set this true so a tagged lowID can get the next reserved slot
		//lastupslotHighID = true;
		//AddLogLine(true,"Added High ID: %s", newclient->GetUserName());
		RemoveFromWaitingQueue(newclient, true);
		theApp.emuledlg->transferwnd->ShowQueueCount(waitinglist.GetCount());
	}
	else
        return false;

        // [ionix] - removed

	if (IsDownloading(newclient))
		return false;

    if(pszReason && thePrefs.GetLogUlDlEvents())
        AddDebugLogLine(false, _T("Adding client to upload list: %s Client: %s"), pszReason, newclient->DbgGetClientInfo());

	if (newclient->HasCollectionUploadSlot() && directadd == NULL){
		ASSERT( false );
		newclient->SetCollectionUploadSlot(false);
	}

	// tell the client that we are now ready to upload
	if (!newclient->socket || !newclient->socket->IsConnected())
	{
		newclient->SetUploadState(US_CONNECTING);
		if (!newclient->TryToConnect(true))
			return false;
		if (!newclient->socket) // Pawcio: BC
			return false;
		newclient->socket->SetPriorityReceive(true); // Pawcio: BC Download
		newclient->socket->SetPrioritySend(true); // Pawcio: BC Upload
	}
	else
	{
		newclient->socket->SetPriorityReceive(true); // Pawcio: BC Download
		newclient->socket->SetPrioritySend(true); // Pawcio: BC Upload
		// Pawcio: PowerShare
		CKnownFile* reqfile = theApp.sharedfiles->GetFileByID(newclient->GetUploadFileID());
		if (reqfile){
			CSafeMemFile data(16+16);
			data.WriteHash16(reqfile->GetFileHash());
			bool send = true;
			if (reqfile->IsPartFile()){
				((CPartFile*)reqfile)->WritePartStatus(&data, newclient);	// SLUGFILLER: hideOS
				send = reqfile->GetHideOS()>=0 ? reqfile->GetHideOS()!=0 : thePrefs.GetHideOvershares()!=0;
			}
			else if (!reqfile->ShareOnlyTheNeed(&data, newclient)) // Wistly SOTN
				if (!reqfile->HideOvershares(&data, newclient))	// Slugfiller: HideOS
					send = false;
			if (send){
				Packet* packet = new Packet(&data);
				packet->opcode = OP_FILESTATUS;
				theStats.AddUpDataOverheadFileRequest(packet->size);
				newclient->socket->SendPacket(packet,true);
			}
			else {
				BYTE* tmp = data.Detach();
				free(tmp);
			}
		}
		// Pawcio: PowerShare
		if (thePrefs.GetDebugClientTCPLevel() > 0)
			DebugSend("OP__AcceptUploadReq", newclient);
		Packet* packet = new Packet(OP_ACCEPTUPLOADREQ,0);
		theStats.AddUpDataOverheadFileRequest(packet->size);
		newclient->socket->SendPacket(packet,true);
		newclient->SetUploadState(US_UPLOADING);
	}
	newclient->SetUpStartTime();
	newclient->ResetSessionUp();

	InsertInUploadingList(newclient, toReserve);

	if (toReserve)
		m_nLastAddToReserved = ::GetTickCount();
	else
		m_nLastStartUpload = ::GetTickCount();
	
	// statistic
	CKnownFile* reqfile = theApp.sharedfiles->GetFileByID((uchar*)newclient->GetUploadFileID());
	if (reqfile)
		reqfile->statistic.AddAccepted();
		
	if (toReserve) 	// Pawcio: BC
		theApp.emuledlg->transferwnd->reservedlistctrl.AddClient(newclient);
	else
		theApp.emuledlg->transferwnd->uploadlistctrl.AddClient(newclient);

	return true;
}

void CUploadQueue::UpdateActiveClientsInfo(DWORD curTick) {
    // Save number of active clients for statistics
	uint32 tempHighest = activeSlots; // Pawcio: BC

    if(thePrefs.GetLogUlDlEvents() && activeSlots > (uint32)uploadinglist.GetSize()) { // Pawcio: BC
        // debug info, will remove this when I'm done.
        //AddDebugLogLine(false, _T("UploadQueue: Error! Throttler has more slots than UploadQueue! Throttler: %i UploadQueue: %i Tick: %i"), theApp.uploadBandwidthThrottler->GetStandardListSize(), uploadinglist.GetSize(), ::GetTickCount());

		if(tempHighest > (uint32)uploadinglist.GetSize()+1) {
        	tempHighest = uploadinglist.GetSize()+1;
		}
    }

    m_iHighestNumberOfFullyActivatedSlotsSinceLastCall = tempHighest;

    // save some data about number of fully active clients
    uint32 tempMaxRemoved = 0;
    while(!activeClients_tick_list.IsEmpty() && !activeClients_list.IsEmpty() && curTick-activeClients_tick_list.GetHead() > 20*1000) {
        activeClients_tick_list.RemoveHead();
	    uint32 removed = activeClients_list.RemoveHead();

        if(removed > tempMaxRemoved) {
            tempMaxRemoved = removed;
        }
    }

	activeClients_list.AddTail(m_iHighestNumberOfFullyActivatedSlotsSinceLastCall);
    activeClients_tick_list.AddTail(curTick);

    if(activeClients_tick_list.GetSize() > 1) {
        uint32 tempMaxActiveClients = m_iHighestNumberOfFullyActivatedSlotsSinceLastCall;
        uint32 tempMaxActiveClientsShortTime = m_iHighestNumberOfFullyActivatedSlotsSinceLastCall;
        POSITION activeClientsTickPos = activeClients_tick_list.GetTailPosition();
        POSITION activeClientsListPos = activeClients_list.GetTailPosition();
        while(activeClientsListPos != NULL && (tempMaxRemoved > tempMaxActiveClients && tempMaxRemoved >= m_MaxActiveClients || curTick - activeClients_tick_list.GetAt(activeClientsTickPos) < 10 * 1000)) {
            DWORD activeClientsTickSnapshot = activeClients_tick_list.GetAt(activeClientsTickPos);
            uint32 activeClientsSnapshot = activeClients_list.GetAt(activeClientsListPos);

            if(activeClientsSnapshot > tempMaxActiveClients) {
                tempMaxActiveClients = activeClientsSnapshot;
            }

            if(activeClientsSnapshot > tempMaxActiveClientsShortTime && curTick - activeClientsTickSnapshot < 10 * 1000) {
                tempMaxActiveClientsShortTime = activeClientsSnapshot;
            }

            activeClients_tick_list.GetPrev(activeClientsTickPos);
            activeClients_list.GetPrev(activeClientsListPos);
        }

        if(tempMaxRemoved >= m_MaxActiveClients || tempMaxActiveClients > m_MaxActiveClients) {
            m_MaxActiveClients = tempMaxActiveClients;
        }

        m_MaxActiveClientsShortTime = tempMaxActiveClientsShortTime;
    } else {
        m_MaxActiveClients = m_iHighestNumberOfFullyActivatedSlotsSinceLastCall;
        m_MaxActiveClientsShortTime = m_iHighestNumberOfFullyActivatedSlotsSinceLastCall;
    }
}

/**
 * Maintenance method for the uploading slots. It adds and removes clients to the
 * uploading list. It also makes sure that all the uploading slots' Sockets always have
 * enough packets in their queues, etc.
 *
 * This method is called approximately once every 100 milliseconds.
 */
void CUploadQueue::Process() 
{
	DWORD curTick = ::GetTickCount();

	UpdateActiveClientsInfo(curTick);

	// Pawcio: BC
	float currMaxUpload = theApp.BC->GetMaxUpload();

	if (currMaxUpload == UNLIMITED)
		MaxUpload = UNLIMITED;
	else
		MaxUpload = currMaxUpload * 1024.0f;
	if (AcceptToReserved())
		AddUpNextClient(_T("Added in ReservedList"), 0, true);
	else if (AcceptNewClient())
		AddUpNextClient(_T("Direct Add"));
	if (!endingTransfer && thePrefs.IsMinimizeOpenedSlots() && 
		!thePrefs.IsCumulateBandwidth() && reserveduploadinglist.GetCount())
		AddUpNextClient(_T("Moving Client from ReservedList to UploadList"));

	clientsrdy = 0;
	endingTransfer=0;

	//WiZaRd - no need to break here, UL rate must be calculated!
	//if (!uploadinglist.GetCount() && !reserveduploadinglist.GetCount())
	//	return;
	// <--- Pawcio: BC

    // The loop that feeds the upload slots with data.
	POSITION pos = uploadinglist.GetHeadPosition();
	POSITION prev_pos = NULL;; // Pawcio: BC
	while(pos != NULL)
	{
		prev_pos = pos;
        // Get the client. Note! Also updates pos as a side effect.
		CUpDownClient* cur_client = uploadinglist.GetNext(pos);
		CClientReqSocket* cur_ClientSocket = cur_client->GetFileUploadSocket();
		if (thePrefs.m_iDbgHeap >= 2)
			ASSERT_VALID(cur_client);
		//It seems chatting or friend slots can get stuck at times in upload.. This needs looked into..
		if (!cur_client->socket || cur_client->GetTimeSinceLastDataSend() > SEC2MS(40))
		{
			RemoveFromUploadQueue(cur_client, _T("Uploading to client without socket? (CUploadQueue::Process)"));
			if(cur_client->Disconnected(_T("CUploadQueue::Process"))){
				delete cur_client;
			}
			continue;
		} else if (cur_client->toRemove && (!cur_ClientSocket || cur_ClientSocket && !cur_ClientSocket->IsBufferFilled() && !cur_ClientSocket->HasBlock())){
			RemoveFromUploadQueue(cur_client, cur_client->reason);
			continue;
		} else {
            cur_client->SendBlockData();
        }
		// Pawcio: BC
		if (cur_ClientSocket && !cur_ClientSocket->IsBusy() && cur_ClientSocket->HasBlock())
			clientsrdy++;
		if (cur_client->toRemove || cur_client->noBlocksToSend && (!endingTransfer || (cur_client->GetWebCacheUpState() == WCUS_NONE && cur_client->GetPeerCacheUpState() == PCUS_NONE)))
			endingTransfer++;
		if (cur_ClientSocket->onReserved){
			uploadinglist.RemoveAt(prev_pos);
			cur_client->SetTimeOnReserved();
			reserveduploadinglist.AddHead(cur_client);
			theApp.emuledlg->transferwnd->uploadlistctrl.RemoveClient(cur_client);
			theApp.emuledlg->transferwnd->reservedlistctrl.AddClient(cur_client);
		}
		// <--- Pawcio: BC
	}

	// Pawcio: BC
	pos = reserveduploadinglist.GetHeadPosition();
	prev_pos = NULL;; // Pawcio: BC
	while(pos != NULL)
	{
		prev_pos = pos;
        // Get the client. Note! Also updates pos as a side effect.
		CUpDownClient* cur_client = reserveduploadinglist.GetNext(pos);
		cur_client->ConsiderReservedTime(); // Pawcio: BC
		CClientReqSocket* cur_ClientSocket = cur_client->GetFileUploadSocket();
		if (thePrefs.m_iDbgHeap >= 2)
			ASSERT_VALID(cur_client);
		//It seems chatting or friend slots can get stuck at times in upload.. This needs looked into..
		if (!cur_client->socket || cur_client->GetTimeSinceLastDataSend() > SEC2MS(40))
		{
			RemoveFromUploadQueue(cur_client, _T("Uploading to client without socket? (CUploadQueue::Process)"));
			if(cur_client->Disconnected(_T("CUploadQueue::Process"))){
				delete cur_client;
			}
			continue;
		} else if (cur_client->toRemove && (!cur_ClientSocket || cur_ClientSocket && !cur_ClientSocket->IsBufferFilled() && !cur_ClientSocket->HasBlock())){
			RemoveFromUploadQueue(cur_client, cur_client->reason);
			continue;
		} else {
            cur_client->SendBlockData();
        }
		if (cur_ClientSocket && !cur_ClientSocket->IsBusy() && cur_ClientSocket->HasBlock())
			clientsrdy++;
		if (cur_client->toRemove || cur_client->noBlocksToSend && (!endingTransfer || (cur_client->GetWebCacheUpState() == WCUS_NONE && cur_client->GetPeerCacheUpState() == PCUS_NONE)))
			endingTransfer++;
		if (!cur_ClientSocket->onReserved){
			cur_client->ConsiderReservedTime();
			reserveduploadinglist.RemoveAt(prev_pos);
			uploadinglist.AddTail(cur_client);
			theApp.emuledlg->transferwnd->reservedlistctrl.RemoveClient(cur_client);
			theApp.emuledlg->transferwnd->uploadlistctrl.AddClient(cur_client);
			m_nLastStartUpload = ::GetTickCount();
		}
	}
	// <--- Pawcio: BC

    // Save used bandwidth for speed calculations
	uploadValues upVal = theApp.uploadBandwidthThrottler->GetNumberOfSentBytesSinceLastCallAndReset(); // Pawcio: BC
	avarage_dr_list.AddTail(upVal.SentBytes);
    m_avarage_dr_sum += upVal.SentBytes;
	// Pawcio: BC
	bandwidthUsed = upVal.bandwidthUsed;
	activeSlots = upVal.activeSlots;
    // uint64 sentBytesOverhead = theApp.uploadBandwidthThrottler->GetNumberOfSentBytesOverheadSinceLastCallAndReset();
	// <--- Pawcio: BC

    avarage_friend_dr_list.AddTail(theStats.sessionSentBytesToFriend);

    // Save time beetween each speed snapshot
    avarage_tick_list.AddTail(curTick);

    // don't save more than 30 secs of data
    while(avarage_tick_list.GetCount() > 3 && !avarage_friend_dr_list.IsEmpty() && ::GetTickCount()-avarage_tick_list.GetHead() > 30*1000) 
	{
   	    m_avarage_dr_sum -= avarage_dr_list.RemoveHead();
        avarage_friend_dr_list.RemoveHead();
        avarage_tick_list.RemoveHead();
    }
};

//Pawcio: BC
bool CUploadQueue::AcceptToReserved(bool lowID)
{
	if (thePrefs.IsBadwolfsUpload()){
		if (!reserveduploadinglist.GetCount())
			return true;
		if (::GetTickCount() - m_nLastAddToReserved < 8000)
			return false;
		if (reserveduploadinglist.GetCount() + uploadinglist.GetCount() > theApp.BC->GetCurrStatsDataUpload() * 1024 / UPLOAD_CHECK_CLIENT_DR + 4)
			return false;
		if ((reserveduploadinglist.GetCount() >= 1 && theApp.BC->GetMaxUpload() < 10) || (reserveduploadinglist.GetCount() >= 3 && theApp.BC->GetMaxUpload() >= 10))
			return false;
		else
			return true;
		// END By BadWolf
	}
	else{
		if (!uploadinglist.GetCount() && !reserveduploadinglist.GetCount())
			return true;
		if (::GetTickCount() - m_nLastAddToReserved < 8000)
			return false;
		if (MaxUpload == UNLIMITED)
			return false;
		if (thePrefs.IsMinimizeOpenedSlots() && (!bandwidthUsed || endingTransfer || lowID)){
			uint16 openedSlots = uploadinglist.GetCount() + reserveduploadinglist.GetCount();
			if (openedSlots > MaxSlots(lowID))
				return false;
			uint16 nMaxSlots = SlotsInRange(lowID);
			if (!nMaxSlots){
				if (!clientsrdy && !reservedclientsrdy && (openedSlots < MaxUpload / (2500 + datarate/250) + 1) && openedSlots <= MaxSlots() && (theApp.BC->GetCurrPing() <= theApp.BC->GetBasePingUp() || theApp.BC->IsMinUpReached()))
					return true;
				return false;
			}
			if (lowID)
				return true;
			if (endingTransfer && openedSlots < 3)
				return true;
			if (theApp.BC->GetCurrPing() > theApp.BC->GetBasePingUp() && !theApp.BC->IsMinUpReached())
				return false;
			uint16 nMaxReadySlots = 2;
			if (openedSlots >= nMaxSlots/2)
				nMaxReadySlots = 1;
			if (clientsrdy + reservedclientsrdy < nMaxReadySlots + endingTransfer)
				return true;
		}
		else if (!thePrefs.IsMinimizeOpenedSlots()){
			if (reserveduploadinglist.GetCount() < endingTransfer || !reserveduploadinglist.GetCount() && !clientsrdy && !thePrefs.IsCumulateBandwidth())
				return true;
			if (thePrefs.IsCumulateBandwidth())
				return AcceptToUpload(true);
		}
	}
	return false;
}

bool CUploadQueue::AcceptNewClient()
{
	if (thePrefs.IsBadwolfsUpload()){
		if (!uploadinglist.GetCount())
			return true;
	}
	else {
		// check if we can allow a new client to start downloading form us
		//Pawcio: BC
		if (/*MaxUpload != UNLIMITED &&*/ (thePrefs.IsMinimizeOpenedSlots() || thePrefs.IsCumulateBandwidth())){ // [ionix] - changed
			if (!uploadinglist.GetCount())
				return true;
			return false;
		}
		else
			return AcceptToUpload();
		//nope
	}
	return false;
}

// Pawcio: BC
bool CUploadQueue::AcceptToUpload(bool countReserved)
{ 
	uint32 currRate = theApp.BC->GetCurrStatsDataUpload() * 1024; //currRate in bytes
	if (::GetTickCount() - m_nLastStartUpload < 1000 && currRate < KiB2B(100) )
		return false;

	float MaxSpeed = theApp.BC->GetMaxUpload();
	//uint32 upPerClient = UPLOAD_CLIENT_DATARATE; // [ionix] - Max ClientUploadrate
	uint32 curUploadSlots = (uint32)uploadinglist.GetCount() + (countReserved ? reserveduploadinglist.GetCount() : 0);

	if (curUploadSlots < MIN_UP_CLIENTS_ALLOWED)
		return true;
	else if (curUploadSlots >= MAX_UP_CLIENTS_ALLOWED)
		return false;
	if (curUploadSlots >= 4 &&
		(curUploadSlots >= (currRate/UPLOAD_CHECK_CLIENT_DR) ||
		curUploadSlots >= ((uint32)MaxSpeed)*1024/UPLOAD_CLIENT_DATARATE))
		return false;

	// if throttler doesn't require another slot, go with a slightly more restrictive method
	//>>> [ionix] - Max ClientUploadrate
	// moved to own function
	/*
	if( MaxSpeed > 20.0f || MaxSpeed == UNLIMITED)
		upPerClient += currRate/43;

	if( upPerClient > 7680 )
		upPerClient = 7680;
	*/
	const uint32 upPerClient = GetUpPerClient();
	//<<< [ionix] - Max ClientUploadrate

	//now the final check
	if (!theApp.BC->IsPingWorking() && !theApp.BC->IsPingPreparing() && MaxSpeed == UNLIMITED )
	{
		uint32 maxCurrRate = (uint32)theApp.BC->GetMaxDataRate() * 1024;
		if (curUploadSlots < (maxCurrRate/upPerClient))
			return true;
	}
	else{
		uint16 nMaxSlots;
		if (MaxSpeed > 12.0f)
			nMaxSlots = (uint16)(((float)(MaxSpeed*1024)) / upPerClient);
		else if (MaxSpeed > 7.0f)
			nMaxSlots = MIN_UP_CLIENTS_ALLOWED + 2;
		else if (MaxSpeed > 3.0f)
			nMaxSlots = MIN_UP_CLIENTS_ALLOWED + 1;
		else
			nMaxSlots = MIN_UP_CLIENTS_ALLOWED;
		//		AddLogLine(true,"maxslots=%u, upPerClient=%u, datarateslot=%u|%u|%u",nMaxSlots,upPerClient,datarate/UPLOAD_CHECK_CLIENT_DR, datarate, UPLOAD_CHECK_CLIENT_DR);

		if ( curUploadSlots < nMaxSlots )
		{
			return true;
		}
	}
	//nope
	return false;
}
// Pawcio: BC Removed

CUploadQueue::~CUploadQueue(){
	if (h_timer)
		KillTimer(0,h_timer);
}

CUpDownClient* CUploadQueue::GetWaitingClientByIP_UDP(uint32 dwIP, uint16 nUDPPort){
	for (POSITION pos = waitinglist.GetHeadPosition();pos != 0;){
		CUpDownClient* cur_client = waitinglist.GetNext(pos);
		if (dwIP == cur_client->GetIP() && nUDPPort == cur_client->GetUDPPort())
			return cur_client;
	}
	return 0;
}

CUpDownClient* CUploadQueue::GetWaitingClientByIP(uint32 dwIP){
	for (POSITION pos = waitinglist.GetHeadPosition();pos != 0;){
		CUpDownClient* cur_client = waitinglist.GetNext(pos);
		if (dwIP == cur_client->GetIP())
			return cur_client;
	}
	return 0;
}

/**
 * Add a client to the waiting queue for uploads.
 *
 * @param client address of the client that should be added to the waiting queue
 *
 * @param bIgnoreTimelimit don't check timelimit to possibly ban the client.
 */
void CUploadQueue::AddClientToQueue(CUpDownClient* client, bool bIgnoreTimelimit)
{
	//This is to keep users from abusing the limits we put on lowID callbacks.
	//1)Check if we are connected to any network and that we are a lowID.
	//(Although this check shouldn't matter as they wouldn't have found us..
	// But, maybe I'm missing something, so it's best to check as a precaution.)
	//2)Check if the user is connected to Kad. We do allow all Kad Callbacks.
	//3)Check if the user is in our download list or a friend..
	//We give these users a special pass as they are helping us..
	//4)Are we connected to a server? If we are, is the user on the same server?
	//TCP lowID callbacks are also allowed..
	//5)If the queue is very short, allow anyone in as we want to make sure
	//our upload is always used.
	if (theApp.IsConnected() 
		&& theApp.IsFirewalled()
		&& !client->GetKadPort()
		&& client->GetDownloadState() == DS_NONE 
		&& !client->IsFriend()
		&& theApp.serverconnect
		&& !theApp.serverconnect->IsLocalServer(client->GetServerIP(),client->GetServerPort())
		&& GetWaitingUserCount() > 50)
		return;

#ifdef WEBCACHE
	// this file is shared but not a single chunk is complete, so don't enqueue the clients asking for it
	CKnownFile* uploadReqfile = theApp.sharedfiles->GetFileByID(client->requpfileid);
	if (uploadReqfile && uploadReqfile->IsPartFile() && ((CPartFile*)uploadReqfile)->GetAvailablePartCount() == 0)
		return;
#endif

	client->AddAskedCount();
	client->SetLastUpRequest();
	if (!bIgnoreTimelimit)
		client->AddRequestCount(client->GetUploadFileID());
	if (client->IsBanned())
		return;

	//Xman start
	//check completed sources which want to download their "complete" file

	if(client->GetRequestFile()==uploadReqfile && client->HasFileComplete())
	{
		AddDebugLogLine(false, _T("->client want to download a file it has already complete: %s, %s"), uploadReqfile->GetFileName(), client->DbgGetClientInfo());
		return; //no answer... a ban would also be not to bad
	}

	//Xman end

	uint16 cSameIP = 0;
	// check for double
	POSITION pos1, pos2;
	for (pos1 = waitinglist.GetHeadPosition();( pos2 = pos1 ) != NULL;)
	{
		waitinglist.GetNext(pos1);
		CUpDownClient* cur_client= waitinglist.GetAt(pos2);
		if (cur_client == client)
		{
			// Pawcio: BC LowID to Reserved	
			if (client->m_bAddNextConnect /*&& lastupslotHighID*/)
				if (thePrefs.IsMinimizeOpenedSlots() && AcceptToReserved(true)
					// [ionix] - This needs an explanation
					// When we have 12 as MaxUpload, we upload to 4 HighID clients
					// This next line will allow up to 8 LowID clients
					// This needs to be limited
					//|| !thePrefs.IsMinimizeOpenedSlots() && (uint32)uploadinglist.GetCount()<<10 < MaxUpload)
					// OK...we allow two lowID slots here
					|| !thePrefs.IsMinimizeOpenedSlots() && (uint32)uploadinglist.GetCount() < (MaxUpload/GetUpPerClient()+2))
			{
				//Special care is given to lowID clients that missed their upload slot
				//due to the saving bandwidth on callbacks.
				if(thePrefs.GetLogUlDlEvents())
					AddDebugLogLine(true, _T("Adding ****lowid when reconnecting. Client: %s"), client->DbgGetClientInfo());
				client->m_bAddNextConnect = false;
				RemoveFromWaitingQueue(client, true);
				AddUpNextClient(_T("Adding ****lowid when reconnecting."), client);
				return;
			}
			// Pawcio: BC
			client->SendRankingInfo();
			theApp.emuledlg->transferwnd->queuelistctrl.RefreshClient(client);
			return;	
		}
		else if ( client->Compare(cur_client) ) 
		{
			theApp.clientlist->AddTrackClient(client); // in any case keep track of this client

			// another client with same ip:port or hash
			// this happens only in rare cases, because same userhash / ip:ports are assigned to the right client on connecting in most cases
			if (cur_client->credits != NULL && cur_client->credits->GetCurrentIdentState(cur_client->GetIP()) == IS_IDENTIFIED)
			{
				//cur_client has a valid secure hash, don't remove him
				if (thePrefs.GetVerbose())
					AddDebugLogLine(false,CString(GetResString(IDS_SAMEUSERHASH)),client->GetUserName(),cur_client->GetUserName(),client->GetUserName() );
				return;
			}
			if (client->credits != NULL && client->credits->GetCurrentIdentState(client->GetIP()) == IS_IDENTIFIED)
			{
				//client has a valid secure hash, add him remove other one
				if (thePrefs.GetVerbose())
					AddDebugLogLine(false,CString(GetResString(IDS_SAMEUSERHASH)),client->GetUserName(),cur_client->GetUserName(),cur_client->GetUserName() );
				RemoveFromWaitingQueue(pos2,true);
				if (!cur_client->socket)
				{
					if(cur_client->Disconnected(_T("AddClientToQueue - same userhash 1")))
						delete cur_client;
				}
			}
			else
			{
				// remove both since we do not know who the bad one is
				if (thePrefs.GetVerbose())
					AddDebugLogLine(false,CString(GetResString(IDS_SAMEUSERHASH)),client->GetUserName(),cur_client->GetUserName(),_T("Both") );
				RemoveFromWaitingQueue(pos2,true);	
				if (!cur_client->socket)
				{
					if(cur_client->Disconnected(_T("AddClientToQueue - same userhash 2")))
						delete cur_client;
				}
				return;
			}
		}
		else if (client->GetIP() == cur_client->GetIP())
		{
			// same IP, different port, different userhash
			cSameIP++;
		}
	}
	if (cSameIP >= 3)
	{
		// do not accept more than 3 clients from the same IP
		if (thePrefs.GetVerbose())
			DEBUG_ONLY( AddDebugLogLine(false,_T("%s's (%s) request to enter the queue was rejected, because of too many clients with the same IP"), client->GetUserName(), ipstr(client->GetConnectIP())) );
		return;
	}
	else if (theApp.clientlist->GetClientsFromIP(client->GetIP()) >= 3)
	{
		if (thePrefs.GetVerbose())
			DEBUG_ONLY( AddDebugLogLine(false,_T("%s's (%s) request to enter the queue was rejected, because of too many clients with the same IP (found in TrackedClientsList)"), client->GetUserName(), ipstr(client->GetConnectIP())) );
		return;
	}
	// done

	// statistic values
	CKnownFile* reqfile = theApp.sharedfiles->GetFileByID((uchar*)client->GetUploadFileID());
	if (reqfile)
		reqfile->statistic.AddRequest();

	// emule collection will bypass the queue
	if (reqfile != NULL && CCollection::HasCollectionExtention(reqfile->GetFileName()) && reqfile->GetFileSize() < MAXPRIORITYCOLL_SIZE
		&& !client->IsDownloading() && client->socket != NULL && client->socket->IsConnected())
	{
		client->SetCollectionUploadSlot(true);
		RemoveFromWaitingQueue(client, true);
		AddUpNextClient(_T("Collection Priority Slot"), client);
		return;
	}
	else
		client->SetCollectionUploadSlot(false);

	// cap the list
	// the queue limit in prefs is only a soft limit. Hard limit is 25% higher, to let in powershare clients and other
    // high ranking clients after soft limit has been reached
    uint32 hardQueueLimit = thePrefs.GetQueueSize();
    uint32 softQueueLimit = thePrefs.GetQueueSize()*0.8f;
    if(hardQueueLimit - softQueueLimit < 200)
		softQueueLimit = hardQueueLimit-200; 

	// if soft queue limit has been reached, only let in high ranking clients
	if ((uint32)waitinglist.GetCount() >= hardQueueLimit ||
	(uint32)waitinglist.GetCount() >= softQueueLimit && // soft queue limit is reached
	(client->IsFriend() && client->GetFriendSlot()) == false && // client is not a friend with friend slot
	client->GetCombinedFilePrioAndCredit() < GetAverageCombinedFilePrioAndCredit()) { // and client has lower credits/wants lower prio file than average client in queue

		// then block client from getting on queue
		return;
	}
	if (client->IsDownloading())
	{
		// he's already downloading and wants probably only another file
		if (thePrefs.GetDebugClientTCPLevel() > 0)
			DebugSend("OP__AcceptUploadReq", client);
		Packet* packet = new Packet(OP_ACCEPTUPLOADREQ,0);
		theStats.AddUpDataOverheadFileRequest(packet->size);
		client->socket->SendPacket(packet,true);
		return;
	}
	// Pawcio: BC
	waitinglist.AddTail(client);
	client->SetUploadState(US_ONUPLOADQUEUE);
	theApp.emuledlg->transferwnd->queuelistctrl.AddClient(client,true);
	theApp.emuledlg->transferwnd->ShowQueueCount(waitinglist.GetCount());
	client->SendRankingInfo();
	// Pawcio: BC
}

float CUploadQueue::GetAverageCombinedFilePrioAndCredit() {
    DWORD curTick = ::GetTickCount();

    if (curTick - m_dwLastCalculatedAverageCombinedFilePrioAndCredit > 5*1000) {
        m_dwLastCalculatedAverageCombinedFilePrioAndCredit = curTick;

        // TODO: is there a risk of overflow? I don't think so...
        double sum = 0;
	    for (POSITION pos = waitinglist.GetHeadPosition(); pos != NULL; /**/){
		    CUpDownClient* cur_client =	waitinglist.GetNext(pos);
             sum += cur_client->GetCombinedFilePrioAndCredit();
        }
        m_fAverageCombinedFilePrioAndCredit = (float)(sum/waitinglist.GetSize());
    }

    return m_fAverageCombinedFilePrioAndCredit;
}

bool CUploadQueue::RemoveFromUploadQueue(CUpDownClient* client, LPCTSTR pszReason, bool updatewindow, bool earlyabort)
{
    bool result = false;
	// Pawcio: BC
	POSITION pos = NULL;
	POSITION posNormal = uploadinglist.Find(client);
	if (posNormal)
		pos = posNormal;
	else 
		pos = reserveduploadinglist.Find(client);
	if (pos)
	{
		if (updatewindow)
		{
			if (posNormal)
				theApp.emuledlg->transferwnd->uploadlistctrl.RemoveClient(client);
			else
				theApp.emuledlg->transferwnd->reservedlistctrl.RemoveClient(client);
		}

		if (thePrefs.GetLogUlDlEvents())
                AddDebugLogLine(DLP_VERYLOW, true,_T("Removing client from upload list: %s Client: %s Transferred: %s SessionUp: %s QueueSessionPayload: %s"), pszReason==NULL ? _T("") : pszReason, client->DbgGetClientInfo(), CastSecondsToHM( client->GetUpStartTimeDelay()/1000), CastItoXBytes(client->GetSessionUp(), false, false), CastItoXBytes(client->GetQueueSessionPayloadUp(), false, false));
            client->m_bAddNextConnect = false;
		if (posNormal)
			uploadinglist.RemoveAt(pos);
		else
			reserveduploadinglist.RemoveAt(pos);
		bool removed = theApp.uploadBandwidthThrottler->RemoveFromStandardList(client->socket);
		bool pcRemoved = theApp.uploadBandwidthThrottler->RemoveFromStandardList((CClientReqSocket*)client->m_pPCUpSocket);
#ifdef WEBCACHE
		bool wcRemoved = theApp.uploadBandwidthThrottler->RemoveFromStandardList((CClientReqSocket*)client->m_pWCUpSocket);
#endif
		(void)removed;
		(void)pcRemoved;
#ifdef WEBCACHE
		(void)wcRemoved;
#endif
		//if(thePrefs.GetLogUlDlEvents() && !(removed || pcRemoved)) {
		//    AddDebugLogLine(false, _T("UploadQueue: Didn't find socket to delete. Adress: 0x%x"), client->socket);
		//}

		if(client->GetSessionUp() > 0) {
			++successfullupcount;
			totaluploadtime += client->GetUpStartTimeDelay()/1000;
		} else if(earlyabort == false)
			++failedupcount;
		else // Pawcio: BC
			m_nLastAddToReserved = m_nLastStartUpload = ::GetTickCount() - 8000;

		CKnownFile* requestedFile = theApp.sharedfiles->GetFileByID(client->GetUploadFileID());

		if(requestedFile != NULL) {
			    //[ionix] - Added by SiRoB, UpdatePartsInfo -Fix-
				if(requestedFile->IsPartFile())
					((CPartFile*)requestedFile)->UpdatePartsInfo();
				else
				//[ionix] - Added by SiRoB, UpdatePartsInfo -Fix-
			requestedFile->UpdatePartsInfo();
		}
		theApp.clientlist->AddTrackClient(client); // Keep track of this client
		client->SetUploadState(US_NONE);
		client->ClearUploadBlockRequests();
		if (client->addOnWaiting)
			AddClientToQueue(client, true);
		client->toRemove = false;
		client->addOnWaiting = false;
		if (client->socket){ 
			// Pawcio: PowerShare
			if (requestedFile){
				CSafeMemFile data(16+16);
				data.WriteHash16(requestedFile->GetFileHash());
				bool send = true;
				if (requestedFile->IsPartFile()){
					((CPartFile*)requestedFile)->WritePartStatus(&data);	// SLUGFILLER: hideOS
					send = requestedFile->GetHideOS()>=0 ? requestedFile->GetHideOS()!=0 : thePrefs.GetHideOvershares()!=0;
				}
				else {
					if (!requestedFile->ShareOnlyTheNeed(&data, client)) // Wistly SOTN
						if (!requestedFile->HideOvershares(&data, client))	// Slugfiller: HideOS
							send = false;
					data.WriteUInt16(0);
				}
				if (send){
					Packet* packet = new Packet(&data);
					packet->opcode = OP_FILESTATUS;
					theStats.AddUpDataOverheadFileRequest(packet->size);
					client->socket->SendPacket(packet,true);
				}
				else {
					BYTE* tmp = data.Detach();
					free(tmp);
				}
			}
			// Pawcio: PowerShare
		}

		m_iHighestNumberOfFullyActivatedSlotsSinceLastCall = 0;

		result = true;
	}
	/*if (posNormal){
		pos = uploadinglist.GetHeadPosition();
		while (pos){
			client = uploadinglist.GetNext(pos);
			client->SetSlotNumber(slotCounter);
			slotCounter++;
		}
	}
	else {
		pos = reserveduploadinglist.GetHeadPosition();
		while (pos){
			client = reserveduploadinglist.GetNext(pos);
			client->SetSlotNumber(slotCounter);
			slotCounter++;
		}
	}*/
	// <--- Pawcio: BC
	return result;
}

uint32 CUploadQueue::GetAverageUpTime(){
	if( successfullupcount ){
		return totaluploadtime/successfullupcount;
	}
	return 0;
}

bool CUploadQueue::RemoveFromWaitingQueue(CUpDownClient* client, bool updatewindow){
	POSITION pos = waitinglist.Find(client);
	if (pos){
		RemoveFromWaitingQueue(pos,updatewindow);
		if (updatewindow)
			theApp.emuledlg->transferwnd->ShowQueueCount(waitinglist.GetCount());
		client->m_bAddNextConnect = false;
		return true;
	}
	else
		return false;
}

void CUploadQueue::RemoveFromWaitingQueue(POSITION pos, bool updatewindow){	
	CUpDownClient* todelete = waitinglist.GetAt(pos);
	waitinglist.RemoveAt(pos);
	if (updatewindow)
		theApp.emuledlg->transferwnd->queuelistctrl.RemoveClient(todelete);
	todelete->SetUploadState(US_NONE);
}

void CUploadQueue::UpdateMaxClientScore()
{
	m_imaxscore=0;
	for(POSITION pos = waitinglist.GetHeadPosition(); pos != 0; ) 
	{
		CUpDownClient* client = waitinglist.GetNext(pos);
		uint32 score = client->GetScore(true, false);
		if(score > m_imaxscore )
			m_imaxscore=score;
	}
}

bool CUploadQueue::CheckForTimeOver(CUpDownClient* client){
	//If we have nobody in the queue, do NOT remove the current uploads..
	//This will save some bandwidth and some unneeded swapping from upload/queue/upload..
	if ( waitinglist.IsEmpty() || client->GetFriendSlot() )
		return false;

	if(client->HasCollectionUploadSlot()){
		CKnownFile* pDownloadingFile = theApp.sharedfiles->GetFileByID(client->requpfileid);
		if(pDownloadingFile == NULL)
			return true;
		if (CCollection::HasCollectionExtention(pDownloadingFile->GetFileName()) && pDownloadingFile->GetFileSize() < MAXPRIORITYCOLL_SIZE)
			return false;
		else{
			if (thePrefs.GetLogUlDlEvents())
				AddDebugLogLine(DLP_HIGH, false, _T("%s: Upload session ended - client with Collection Slot tried to request blocks from another file"), client->GetUserName());
			return true;
		}
	}

        // [ionix] - removed

	// Allow the client to download a specified amount per session
	if( client->GetQueueSessionPayloadUp() > SESSIONMAXTRANS ){
		if (thePrefs.GetLogUlDlEvents())
			AddDebugLogLine(DLP_DEFAULT, false, _T("%s: Upload session ended due to max transferred amount. %s"), client->GetUserName(), CastItoXBytes(SESSIONMAXTRANS, false, false));
		return true;
	}

	return false;
}

void CUploadQueue::DeleteAll(){
	waitinglist.RemoveAll();
	uploadinglist.RemoveAll();
	reserveduploadinglist.RemoveAll(); // Pawcio: BC
    // PENDING: Remove from UploadBandwidthThrottler as well!
}

uint16 CUploadQueue::GetWaitingPosition(CUpDownClient* client)
{
	if (!IsOnUploadQueue(client))
		return 0;
	UINT rank = 1;
	uint32 myscore = client->GetScore(false);
	for (POSITION pos = waitinglist.GetHeadPosition(); pos != 0; ){
		if (waitinglist.GetNext(pos)->GetScore(false) > myscore)
			rank++;
	}
	return rank;
}

VOID CALLBACK CUploadQueue::UploadTimer(HWND hwnd, UINT uMsg,UINT_PTR idEvent,DWORD dwTime)
{
	// NOTE: Always handle all type of MFC exceptions in TimerProcs - otherwise we'll get mem leaks
	try
	{
		// Barry - Don't do anything if the app is shutting down - can cause unhandled exceptions
		if (!theApp.emuledlg->IsRunning())
			return;

        // Elandal:ThreadSafeLogging -->
        // other threads may have queued up log lines. This prints them.
        theApp.HandleDebugLogQueue();
        theApp.HandleLogQueue();
        // Elandal: ThreadSafeLogging <--

		theApp.pNAFC->Process(); //>>> WiZaRd::NAFC
#ifdef SPOOKY
//WiZaRd: I don't know why this got removed?! Without these lines the conchecker is totally useless!
        const DWORD timer = ::GetTickCount();
		theApp.conchecker.Process(timer);
#endif

        // Send allowed data rate to UploadBandWidthThrottler in a thread safe way
		// Pawcio: BC Removed

		theApp.BC->Process(); // Pawcio: BC
        theApp.uploadqueue->Process();
		theApp.downloadqueue->ProcessReceiving();
		if (thePrefs.ShowOverhead()){
			theStats.CompUpDatarateOverhead();
			theStats.CompDownDatarateOverhead();
		}
		counter++;

		// 500 ms
		if (counter % 10 == 1 || counter % 10 == 6){
			theApp.downloadqueue->Process();
			theApp.listensocket->SetLimitForConnections(); //Pawcio: BC Experimental Socket Overhead
			if (theApp.emuledlg->IsUpdateBCNeeded()){
				theApp.emuledlg->UpdateBCStatus(false);
				theApp.emuledlg->preferenceswnd->UpdateBCStatus();
			}
		}

		// one second
		if (counter >= 10){
			counter=0;

			// try to use different time intervals here to not create any disk-IO bottle necks by saving all files at once
			theApp.clientcredits->Process();	// 13 minutes
			theApp.serverlist->Process();		// 17 minutes
			theApp.knownfiles->Process();		// 11 minutes
			theApp.friendlist->Process();		// 19 minutes
			theApp.clientlist->Process();
			theApp.sharedfiles->Process();
			if( Kademlia::CKademlia::isRunning() )
			{
				Kademlia::CKademlia::process();
				if(Kademlia::CKademlia::getPrefs()->hasLostConnection())
				{
					Kademlia::CKademlia::stop();
					theApp.emuledlg->ShowConnectionState();
				}
			}
			if( theApp.serverconnect->IsConnecting() && !theApp.serverconnect->IsSingleConnect() )
				theApp.serverconnect->TryAnotherConnectionrequest();

			theApp.listensocket->UpdateConnectionsStatus();
			if (thePrefs.WatchClipboard4ED2KLinks())
				theApp.SearchClipboard();		

			if (theApp.serverconnect->IsConnecting())
				theApp.serverconnect->CheckForTimeout();

			// -khaos--+++> Update connection stats...
			iupdateconnstats++;
			// 2 seconds
			if (iupdateconnstats>=2) {
				iupdateconnstats=0;
				theStats.UpdateConnectionStats(theApp.BC->GetCurrStatsDataUpload(), theApp.BC->GetCurrStatsDataDownload()); // Pawcio: BC
			}
			// <-----khaos-

			// display graphs
			if (thePrefs.GetTrafficOMeterInterval()>0) {
				igraph++;

				if (igraph >= (uint32)(thePrefs.GetTrafficOMeterInterval()) ) {
					igraph=0;
					//theApp.emuledlg->statisticswnd->SetCurrentRate((float)(theApp.uploadqueue->Getavgupload()/theApp.uploadqueue->Getavg())/1024,(float)(theApp.uploadqueue->Getavgdownload()/theApp.uploadqueue->Getavg())/1024);
					theApp.emuledlg->statisticswnd->SetCurrentRate(theApp.BC->GetCurrStatsDataUpload(),theApp.BC->GetCurrStatsDataDownload(), theApp.BC->GetCurrStatsUpload(), theApp.BC->GetCurrStatsDownload()); // Pawcio: BC
					//theApp.uploadqueue->Zeroavg();
				}
			}
			if (theApp.emuledlg->activewnd == theApp.emuledlg->statisticswnd && theApp.emuledlg->IsWindowVisible() )  {
				// display stats
				if (thePrefs.GetStatsInterval()>0) {
					istats++;

					if (istats >= (uint32)(thePrefs.GetStatsInterval()) ) {
						istats=0;
						theApp.emuledlg->statisticswnd->ShowStatistics();
					}
				}
			}

            theApp.uploadqueue->UpdateDatarates();
            
			//save rates every second
			theStats.RecordRate();
			// mobilemule sockets
			theApp.mmserver->Process();

			// [ionix] - TBH: minimule
			if (theApp.minimule->IsWindowVisible() || thePrefs.GetMiniMuleLives()) theApp.minimule->RunMiniMule();
			// [ionix] - TBH: minimule

			// ZZ:UploadSpeedSense -->
            theApp.emuledlg->ShowPing();

            bool gotEnoughHosts = theApp.clientlist->GiveClientsForTraceRoute();
            if(gotEnoughHosts == false) {
                theApp.serverlist->GiveServersForTraceRoute();
            }
			// ZZ:UploadSpeedSense <--

            sec++;
			// 5 seconds
			if (sec>=5) {
#ifdef _DEBUG
				if (thePrefs.m_iDbgHeap > 0 && !AfxCheckMemory())
					AfxDebugBreak();
#endif
//>>> WiZaRd::Recheck Firewalled
				if( m_uiFirewalledTimer < ::GetTickCount() )
				{
					m_uiFirewalledTimer = ::GetTickCount()+MIN2MS(2);
					if( Kademlia::CKademlia::isConnected() && Kademlia::CKademlia::isFirewalled())
						Kademlia::CKademlia::RecheckFirewalled();					
				}
//<<< WiZaRd::Recheck Firewalled
				sec = 0;
				theApp.listensocket->Process();
				theApp.OnlineSig(); // Added By Bouc7 
				theApp.emuledlg->ShowTransferRate();

                                // [ionix] - removed

				// update cat-titles with downloadinfos only when needed
				if (thePrefs.ShowCatTabInfos() && 
					theApp.emuledlg->activewnd == theApp.emuledlg->transferwnd && 
					theApp.emuledlg->IsWindowVisible()) 
						theApp.emuledlg->transferwnd->UpdateCatTabTitles(false);
				
				if (thePrefs.IsSchedulerEnabled())
					theApp.scheduler->Check();

                theApp.emuledlg->transferwnd->UpdateListCount(CTransferWnd::wnd2Uploading, -1);
			}

			statsave++;
			// 60 seconds
			if (statsave>=60) {
				statsave=0;

				if (thePrefs.GetWSIsEnabled())
					theApp.webserver->UpdateSessionCount();

				theApp.serverconnect->KeepConnectionAlive();
			}

			_uSaveStatistics++;
			if (_uSaveStatistics >= thePrefs.GetStatsSaveInterval())
			{
				_uSaveStatistics = 0;
				thePrefs.SaveStats();
			}
		}

		// need more accuracy here. don't rely on the 'sec' and 'statsave' helpers.
		thePerfLog.LogSamples();
	}
	CATCH_DFLT_EXCEPTIONS(_T("CUploadQueue::UploadTimer"))
}

CUpDownClient* CUploadQueue::GetNextClient(const CUpDownClient* lastclient){
	if (waitinglist.IsEmpty())
		return 0;
	if (!lastclient)
		return waitinglist.GetHead();
	POSITION pos = waitinglist.Find(const_cast<CUpDownClient*>(lastclient));
	if (!pos){
		TRACE("Error: CUploadQueue::GetNextClient");
		return waitinglist.GetHead();
	}
	waitinglist.GetNext(pos);
	if (!pos)
		return NULL;
	else
		return waitinglist.GetAt(pos);
}

void CUploadQueue::UpdateDatarates() {
    // Calculate average datarate
    if(::GetTickCount()-m_lastCalculatedDataRateTick > 500) {
        m_lastCalculatedDataRateTick = ::GetTickCount();

        if(avarage_dr_list.GetSize() >= 2 && (avarage_tick_list.GetTail() > avarage_tick_list.GetHead())) {
	        datarate = (UINT)(((m_avarage_dr_sum - avarage_dr_list.GetHead())*1000) / (avarage_tick_list.GetTail() - avarage_tick_list.GetHead()));
            friendDatarate = (UINT)(((avarage_friend_dr_list.GetTail() - avarage_friend_dr_list.GetHead())*1000) / (avarage_tick_list.GetTail() - avarage_tick_list.GetHead()));
        }
    }
}
float CUploadQueue::GetDatarate() 
{
	return theApp.BC->GetCurrStatsDataUpload(); // Pawcio BC
}

uint32 CUploadQueue::GetToNetworkDatarate() 
{
    if(datarate > friendDatarate) 
        return datarate - friendDatarate;
    else 
        return 0;    
}

void CUploadQueue::ReSortUploadSlots(CUpDownClient* client) {
	// Pawcio: BC
	if (client)
		theApp.uploadBandwidthThrottler->ReSortUploadSlots(client, uploadinglist.Find(client) ? false : true);
}

// Pawcio: BC
uint16 CUploadQueue::SlotsInRange(bool lowID)
{
	uint32 currRate = theApp.BC->GetCurrStatsDataUpload() * 1024;
	uint32 curUploadSlots = (uint32)uploadinglist.GetCount() + (uint32)reserveduploadinglist.GetCount();
	if (curUploadSlots >= 4 &&
		(curUploadSlots >= (currRate/UPLOAD_CHECK_CLIENT_DR) ||
		curUploadSlots >= ((uint32)MaxUpload/UPLOAD_CLIENT_DATARATE)))
		return false;

	uint32 sendPerClient = UPLOAD_CLIENT_DATARATE + MaxUpload/250;

	if( sendPerClient > 4000 )
		sendPerClient = 4000;
	uint16 retVal;
	if (curUploadSlots < (retVal = MIN_UP_CLIENTS_ALLOWED))
		return retVal;
	/*uint16 additionalSlots = 1;
	if (MaxUpload > 8000)
	additionalSlots = log10f(MaxUpload/1000.0f)/0.301f - 1;*/
	if (curUploadSlots < (retVal = (uint16)(MaxUpload/sendPerClient) + /*additionalSlots +*/ (uint16)lowID))
		return retVal;
	return 0;
}

uint16 CUploadQueue::MaxSlots(bool lowID){
	uint32 maxCurrRate = (uint32)theApp.BC->GetMaxDataRate() * 1024;
	//<<< [ionix] - Max ClientUploadrate
	/*
	uint32 maxSendPerClient = UPLOAD_CLIENT_DATARATE + maxCurrRate/250;
	if (maxSendPerClient > 4000)
		maxSendPerClient = 4000;
	uint16 retVal = (uint16)(maxCurrRate / maxSendPerClient) + (uint16)lowID;
	*/
	uint16 retVal = (uint16)(maxCurrRate / GetUpPerClient()) + (uint16)lowID;
	//>>> [ionix] - Max ClientUploadrate
	if (retVal < MIN_UP_CLIENTS_ALLOWED)
		retVal = MIN_UP_CLIENTS_ALLOWED;
	return retVal;
}
//Pawcio: BC

#ifdef WEBCACHE
CUpDownClient*	CUploadQueue::FindClientByWebCacheUploadId(const uint32 id) // Superlexx - webcache - can be made more efficient
{
	if (uploadinglist.IsEmpty())
		return NULL;

	for (POSITION pos = uploadinglist.GetHeadPosition(); pos != NULL;)
	{
		CUpDownClient* cur_client = uploadinglist.GetNext(pos);
		if ( cur_client->m_uWebCacheUploadId == id )
			return cur_client;
	}
	return NULL;
}
#endif